#python library
import os
import re
from StringIO import StringIO
from struct import unpack
import math

import numpy

DICOMTags = {	'subjectName': "0x0010,0x0010",
				'subjectDOB': "0x0010,0x0030",
				'subjectSex': "0x0010,0x0040",
				'subjectAddress': "0x0010,0x1040",
				'imageNumber': "0x0020,0x0013",
				'seriesNumber': "0x0020,0x0011", 
				'seriesTime': "0x0008,0x0031", 
				'imageType': "0x0008,0x0008",
				'acquisitionDate': "0x0008,0x0020",
				'seriesDescription': "0x0008,0x103e",
				'seriesTime': "0x0008,0x0031",
				'slicePosition': "0x0020,0x0032",
				'numRows': "0x0028,0x0010",
				'numCols': "0x0028,0x0011",
				'bitsAllocated': "0x0028,0x0100",
				'samplesPerPixel': "0x0028,0x0002",
				'acquisitionTime': "0x0008,0x0032",
				'acquisitionModality': "0x0008,0x0060",
				'scannerManufacturer': "0x0008,0x0070",
				'institutionName': "0x0008,0x0080", # used for guessing the scanner manufacturer
				'pixelSpacing': "0x0028,0x0030",
				'sliceOrientation': "0x0020,0x0037",
				'acquisitionMatrix': "0x0018,0x1310",
				'FOV': "0x0051,0x100c",
				'TR': "0x0018,0x0080",
				'TE': "0x0018,0x0081",
				'MRFieldStrength': "0x0018,0x0087",
				'flipAngle': "0x0018,0x1314",
				'numAverages': "0x0018,0x0083",
				'BValue': "0x0019,0x100c",
				'BVector': "0x0019,0x100e",
				'numSlices': "0x0019,0x100a",
				'spacingBetweenSlices': "0x0018,0x0088",
				'echoNumber': "0x0018,0x0086",
				'sliceThickness': "0x0018,0x0050",
				'CSAImageHeader': "0x0029,0x1010",
				'CSAHeaderFormat': "0x0029,0x0010"}

DICOMMetaDataTags = {
						'fileMetaInformationGroupLength': "0x0002,0x0000",
						'fileMetaInformationVersion': "0x0002,0x0001",
						'mediaStorageSOPClassUID': "0x0002,0x0002",
						'mediaStorageSOPInstanceUID': "0x0002,0x0003",
						'transferSyntaxUID': "0x0002,0x0010",
						'implementationClassUID': "0x0002,0x0012",
						'implementationVersionName': "0x0002,0x0013"}


# keep in mind for GE scanners
#const gdcm::DictEntry GEDictBValue( 0x0043, 0x1039, "IS", "1", "B Value of diffusion weighting" );
#const gdcm::DictEntry GEDictXGradient( 0x0019, 0x10bb, "DS", "1", "X component of gradient direction" );
#const gdcm::DictEntry GEDictYGradient( 0x0019, 0x10bc, "DS", "1", "Y component of gradient direction" );
#const gdcm::DictEntry GEDictZGradient( 0x0019, 0x10bd, "DS", "1", "Z component of gradient direction" );

#pydicom
import dicom

# this project
import DataTransferProject
#from DataTransferProject import *
import DataTransferConstants

import Exceptions

# sets the major fields in the 
class DICOMDataResource(DataTransferProject.DataResource):
	def __init__(self, **kwargs):
		super(DICOMDataResource, self).__init__(**kwargs) # **kwargs passes through all the arguments that were specified using keyword, value pairs

	def getFileFormat(self):
		return "DICOM"
	
	def anonymiseFiles(self, temporaryDirectory = None, subjectNameEnc = None, subjectAddressEnc = None):
		(fileList, sizes, offsets) = self.getFileListSizesAndOffsets()
		#print subjectNameEnc
		#print subjectAddressEnc
		returnedFileList = []
		for curFile in fileList:
			(head, tail) = os.path.split(curFile)
			fp = file(os.path.join(temporaryDirectory, "." + tail), 'w')

			fp.write(subjectNameEnc + "\n")
			fp.write(subjectAddressEnc + "\n")
			fp.close()

			ds = dicom.filereader.ReadFile(curFile)
			if not (0x10,0x10) in ds:
				ds.AddNew(0x00100010, "PN", "")
			else:
				ds[0x10,0x10].value = "" # subject name

			if not (0x10,0x1040) in ds:
				ds.AddNew(0x00101040, "LO", "")
			else:
				ds[0x10,0x1040].value = "" # subject address
			#dicom.filewriter.WriteFile(os.path.join(databaseDirectory, curAcquisitionURI, tail), ds)
			dicom.filewriter.WriteFile(os.path.join(temporaryDirectory, tail), ds)

			del ds
			returnedFileList.append(tail)
		return returnedFileList

# sorts DICOM files into series and DataResources so that each DataResource
class DICOMFileReader(DataTransferProject.FileReader):
	def __init__(self):
		super(DICOMFileReader, self).__init__()
		self.__DICOMTags = {} # this stores the DICOM tags by file
		self.__subjects = {} # this is used to sort the DICOM files into series

	# we need to be able to let the project
	def getSubjectDetails(self):
		return self.subjectDetails

	def addFile(self, curFile = None, species = None):
		global DICOMTags
		(head, tail) = os.path.split(curFile)
		if tail.lower() != 'dicomdir':
			#ds = dicom.filereader.ReadHeaderSelectedTags(curFile, keyArray = DICOMTags)
			ds = dicom.filereader.read_file(curFile, stop_before_pixels = False)
			if ds != None:
				keyValues = DICOMUtilities.convertDICOMKeysFromDStoDict(ds = ds, keyArray = keyArray)
				if keyValues != None:
					self.__DICOMTags[curFile] = keyValues
					del keyValues
				del ds
			else:
				return None
		else:
			return None
		
		# check to see if critical tags are in the file, if they are not, delete the file, I'm not processing it
		if self.__DICOMTags[curFile]['subjectName'] != None and \
			self.__DICOMTags[curFile]['subjectDOB'] != None and \
			self.__DICOMTags[curFile]['subjectSex'] != None and \
			self.__DICOMTags[curFile]['acquisitionDate'] != None and \
			self.__DICOMTags[curFile]['numRows'] != None and \
			self.__DICOMTags[curFile]['numCols'] != None and \
			self.__DICOMTags[curFile]['bitsAllocated'] != None and \
			self.__DICOMTags[curFile]['imageNumber'] != None and \
			self.__DICOMTags[curFile]['seriesTime'] != None and \
			self.__DICOMTags[curFile]['echoNumber'] != None and \
			self.__DICOMTags[curFile]['seriesNumber'] != None:
			
			subjectHash = DataTransferConstants.getSubjectHash(self.__DICOMTags[curFile]['subjectName'], self.__DICOMTags[curFile]['subjectDOB'])

			if not subjectHash in self.__subjects:
				if self.__DICOMTags[curFile]['subjectName'] == None:
					subjectName = ""
				else:
					subjectName = self.__DICOMTags[curFile]['subjectName']
				if self.__DICOMTags[curFile]['subjectAddress'] == None:
					subjectAddress = ""
				else:
					subjectAddress = self.__DICOMTags[curFile]['subjectAddress']
				self.__subjects[subjectHash] = {}
				self.subjectDetails[subjectHash] = {"DOB": self.__DICOMTags[curFile]['subjectDOB'], "species": "human", "name": self.__DICOMTags[curFile]['subjectName'], "address": self.__DICOMTags[curFile]['subjectAddress'], "hash": subjectHash, "sex": self.__DICOMTags[curFile]['subjectSex']}

			if not self.__DICOMTags[curFile]['acquisitionDate'] in self.__subjects[subjectHash]:
				self.__subjects[subjectHash][self.__DICOMTags[curFile]['acquisitionDate']] = {}
			
			if not (self.__DICOMTags[curFile]['seriesNumber'], self.__DICOMTags[curFile]['seriesTime'], self.__DICOMTags[curFile]['echoNumber']) in self.__subjects[subjectHash][self.__DICOMTags[curFile]['acquisitionDate']]:
				self.__subjects[subjectHash][self.__DICOMTags[curFile]['acquisitionDate']][(self.__DICOMTags[curFile]['seriesNumber'], self.__DICOMTags[curFile]['seriesTime'], self.__DICOMTags[curFile]['echoNumber'])] = {}
			
			if not self.__DICOMTags[curFile]['imageType'] in self.__subjects[subjectHash][self.__DICOMTags[curFile]['acquisitionDate']][(self.__DICOMTags[curFile]['seriesNumber'], self.__DICOMTags[curFile]['seriesTime'], self.__DICOMTags[curFile]['echoNumber'])]:
				self.__subjects[subjectHash][self.__DICOMTags[curFile]['acquisitionDate']][(self.__DICOMTags[curFile]['seriesNumber'], self.__DICOMTags[curFile]['seriesTime'], self.__DICOMTags[curFile]['echoNumber'])][self.__DICOMTags[curFile]['imageType']] = {}
				
			if not self.__DICOMTags[curFile]['imageNumber'] in self.__subjects[subjectHash][self.__DICOMTags[curFile]['acquisitionDate']][(self.__DICOMTags[curFile]['seriesNumber'], self.__DICOMTags[curFile]['seriesTime'], self.__DICOMTags[curFile]['echoNumber'])][self.__DICOMTags[curFile]['imageType']]:
				self.__subjects[subjectHash][self.__DICOMTags[curFile]['acquisitionDate']][(self.__DICOMTags[curFile]['seriesNumber'], self.__DICOMTags[curFile]['seriesTime'], self.__DICOMTags[curFile]['echoNumber'])][self.__DICOMTags[curFile]['imageType']][self.__DICOMTags[curFile]['imageNumber']] = curFile

			#if subjectName != None and subjectDOB != None and subjectSex != None and self.__DICOMTags[curFile]['acquisitionDate'] != None:

			self.subjectDetails[subjectHash]['species'] = species
			return True
		else:
			print curFile + " invalid"
			del self.__DICOMTags[curFile]
			return False
	#def addFile(self, curFile = None, species = None):

	# we need the project because we need to be able to 
	def makeDataResources(self, subjectType = None, studyID = None):
	#	if not isinstance(subjectHashIDMapping, dict):
	#		raise InvalidType()
		
		dataResources = []
		for curSubject in self.__subjects.iterkeys():
			#subjectID = subjectHashIDMapping[curSubject]

			for curAcquisitionDate in self.__subjects[curSubject].iterkeys():
				visitID = curAcquisitionDate

				print "\tvisit: " + visitID
				for curSeries in self.__subjects[curSubject][curAcquisitionDate].iterkeys():
					#print curSubject + " " + curAcquisitionDate + " " + curDirectory + " " + curSeries + " " + str(numImages) + " images"
					for curImageType in self.__subjects[curSubject][curAcquisitionDate][curSeries].iterkeys():
						numImages = len(self.__subjects[curSubject][curAcquisitionDate][curSeries][curImageType])
						mosaicFormat = False
						DTIImage = False
						BValues = []
						BVectors = []
						sliceOrdering = 'unknown'
						
						curImageIdx = 1
						fileList = []
						offsets = []
						sizes = []
						curDataResource = DICOMDataResource(subjectType = subjectType, visitID = visitID, studyID = studyID)

						for curImage in sorted(self.__subjects[curSubject][curAcquisitionDate][curSeries][curImageType].iterkeys()):
							curFile = self.__subjects[curSubject][curAcquisitionDate][curSeries][curImageType][curImage]
							fileList.append(curFile)

							if curImageIdx == 1:
								# set the age at scan field
								curDataResource.setVisitField("subjectAgeAtVisit", str(DataTransferConstants.getAgeAtVisit(visitID, self.__DICOMTags[curFile]['subjectDOB'])))
								#print str(DataTransferConstants.getAgeAtVisit(visitID, self.__DICOMTags[curFile]['subjectDOB']))
								curDataResource.setDataResourceID(self.__DICOMTags[curFile]['imageType'])
								if self.__DICOMTags[curFile]['scannerManufacturer'] == None:
									print curFile + " no scanner manufacturer, guessing from institution name"
									
									if self.__DICOMTags[curFile]['institutionName'] == None:
										print "No institution name either, unknown scanner manufacturer"
									else:
										# if the instutition has "children" in it, assume that it is the royal children's in which case they use siemens scanners
										if self.__DICOMTags[curFile]['institutionName'].lower().find('children') != -1:
											self.__DICOMTags[curFile]['scannerManufacturer'] = "SIEMENS"
										else:
											print "Could not guess scanner manufacturer from institution name"
								else:
									self.__DICOMTags[curFile]['scannerManufacturer'] = self.__DICOMTags[curFile]['scannerManufacturer'].upper()
								curDataResource.setAcquisitionField(key = 'scannerManufacturer', value = self.__DICOMTags[curFile]['scannerManufacturer'])

								#self.__DICOMTags[curFile]['seriesDescription'] = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0008", "0x103e", curFile)
								# deblank and replace reserved characters in unix
								self.__DICOMTags[curFile]['seriesDescription'] = self.__DICOMTags[curFile]['seriesDescription'].replace(" ", "")
								self.__DICOMTags[curFile]['seriesDescription'] = self.__DICOMTags[curFile]['seriesDescription'].replace("/", "_")
								self.__DICOMTags[curFile]['seriesDescription'] = self.__DICOMTags[curFile]['seriesDescription'].replace("<", "_")
								self.__DICOMTags[curFile]['seriesDescription'] = self.__DICOMTags[curFile]['seriesDescription'].replace(">", "_")
								self.__DICOMTags[curFile]['seriesDescription'] = self.__DICOMTags[curFile]['seriesDescription'].replace("(", "_")
								self.__DICOMTags[curFile]['seriesDescription'] = self.__DICOMTags[curFile]['seriesDescription'].replace(")", "_")
								episodeID = self.__DICOMTags[curFile]['seriesDescription']
								curDataResource.setEpisodeID(episodeID)
								print "\t\t" + str(curSeries) + ", series description: " + self.__DICOMTags[curFile]['seriesDescription']	
							
							totalNumRows = self.__DICOMTags[curFile]['numRows']
							totalNumCols = self.__DICOMTags[curFile]['numCols']
							
							#samplesPerPixel = self.__DICOMTags[curFile]['samplesPerPixel']
							
							imageSize = totalNumRows * totalNumCols * self.__DICOMTags[curFile]['bitsAllocated'] / 8 * self.__DICOMTags[curFile]['samplesPerPixel']
							imageOffset = os.path.getsize(curFile) - imageSize
							offsets.append(imageOffset)
							sizes.append(imageSize)
							if self.__DICOMTags[curFile]['acquisitionTime'] != None:
								acquisitionTime = float(self.__DICOMTags[curFile]['acquisitionTime'])
							else:
								acquisitionTime = float(0)
								

							if curImageIdx == 1:
								# work out the format and dimensions of the image and add it to the database
								# the major part of the work goes here
								#details = getDICOMKeyDCKEY.getDICOMMultipleKeyDCKEY(["0x0008,0x0008", "0x0020,0x0032"], curFile)
								firstSlicePosition = self.__DICOMTags[curFile]['slicePosition']
								
								# check to see if the image is a "SECONDARY" or "DERIVED" image, these are not stored as raw data
								DimensionlessData = False

								curT = self.__DICOMTags[curFile]['imageType'].split("\\")
								for curIdx in curT:
									if curIdx == "SECONDARY" or curIdx == "DERIVED":
										DimensionlessData = True
								if DimensionlessData == False:
									# get the slice orientation
									if self.__DICOMTags[curFile]['sliceOrientation'] != None:
										#sliceOrientationElements = self.__DICOMTags[curFile]['sliceOrientation'].split("\\")
										#sliceOrientationX = sliceOrientationElements[:3]
										#sliceOrientationY = sliceOrientationElements[-3:]
										sliceOrientationX = self.__DICOMTags[curFile]['sliceOrientation'][0:3]
										sliceOrientationY = self.__DICOMTags[curFile]['sliceOrientation'][3:6]
										for curElementX in range(len(sliceOrientationX)):
											sliceOrientationX[curElementX] = float(sliceOrientationX[curElementX])
										for curElementY in range(len(sliceOrientationY)):
											sliceOrientationY[curElementY] = float(sliceOrientationY[curElementY])

										sliceOrientationZ = []
										sliceOrientationZ.append(sliceOrientationX[1] * sliceOrientationY[2] - sliceOrientationX[2] * sliceOrientationY[1])
										sliceOrientationZ.append(sliceOrientationX[2] * sliceOrientationY[0] - sliceOrientationX[0] * sliceOrientationY[2])
										sliceOrientationZ.append(sliceOrientationX[0] * sliceOrientationY[1] - sliceOrientationX[1] * sliceOrientationY[0])
									else:
										sliceOrientationX = [float(1), float(0), float(0)]
										sliceOrientationY = [float(0), float(1), float(0)]
										sliceOrientationZ = [float(0), float(0), float(1)]

								# find out the number of slices and the dimensions
								if self.__DICOMTags[curFile]['imageType'].find("MOSAIC") != -1 and self.__DICOMTags[curFile]['scannerManufacturer'].find('SIEMENS') != -1:
									mosaicFormat = True
									numVolumes = numImages
									# this is the volume that we take the second time point from
									# since this is a mosaic format file, the time spacing is the time between the first and second volumes
									secondTimePointVolume = 2
									# for the siemens mosaic images, we need to extract the number of rows and columns
									# by dividing the FOV by the readout and phase FOVs
									# these are available as text fields in the DICOM header
									# they are read by getting the output of the 'strings' command on the DICOM file
									#curPipe = os.popen("strings \"" + curFile + "\"", 'r')
									#DICOMStrings = curPipe.readlines()
									#curPipe.close()
									fp = file(curFile, 'r')
									readFOV = None
									phaseFOV = None
									countedSlices = 0
									for curLine in fp.xreadlines():
										curLine = curLine.strip()
										if curLine.find("sSliceArray.ucMode") != -1:
											pat = re.compile("sSliceArray\.ucMode\s+=\s+(0x[0-9a-fA-F]+)");
											mat = pat.match(curLine)
											if mat != None:
												sliceOrdering = int(mat.group(1), 16)
												if sliceOrdering == 1:
													sliceOrdering = "ascending"
												elif sliceOrdering == 2:
													sliceOrdering = "descending"
												elif sliceOrdering == 4:
													sliceOrdering = "interleaved"
												else:
													sliceOrdering = "unknown"
											else:
												sliceOrdering = "unknown"

										if curLine.find("sSliceArray.asSlice[0].dPhaseFOV") != -1:
											pat = re.compile("sSliceArray\.asSlice\[0\]\.dPhaseFOV\s+=\s+(\d+)");
											mat = pat.match(curLine)
											if mat != None:
												phaseFOV = int(mat.group(1))
											else:
												#print "\"strings\" has stuffed up, waiting for proper line"
												pass
										#if curLine.find("sSliceArray.asSlice[0].dReadFOV") != -1:
										#if curLine.find("sSliceArray.asSlice[0].dPhaseFOV") != -1:
										if curLine.find("sSliceArray.asSlice[0].dReadoutFOV") != -1:
											pat = re.compile("sSliceArray\.asSlice\[0\]\.dReadoutFOV\s+=\s+(\d+)");
											mat = pat.match(curLine)
											if mat != None:
												readoutFOV = int(mat.group(1))
											else:
												pass
												#print "\"strings\" has stuffed up, waiting for proper line"
										#if curLine.find("sSliceArray.asSlice[0].dReadFOV") != -1:
										if self.__DICOMTags[curFile]['FOV'] == None:
											if re.match("sSliceArray.asSlice\[\d+\].dReadoutFOV", curLine) != None:
												countedSlices = countedSlices + 1
										#if readFOV != None and phaseFOV != None:
										#	break
									#OV = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0051", "0x100c", curFile)
									fp.close()
									#print " numRows: " + str(numRows) + ", numCols: " + str(numCols) + " " + curFile
									# if the FOV is missing then we will work out how many slices there are by counting the number of lines that contain sSliceArray.asSlice[\d+].dReadoutFOV in the file
									# then the number of mosaic rows and columns will be the square root of this number
									if self.__DICOMTags[curFile]['FOV'] == None:
										mosaicRows = int(math.ceil(math.sqrt(countedSlices)))
										mosaicCols = mosaicRows
										if mosaicRows != 0:
											numRows = totalNumRows / mosaicRows
											numCols = totalNumCols / mosaicCols
										else:
											# the SIEMENS ASCDATA IS MISSING
											# use acquisition matrix
											acqMatrixNumRows = max(DICOMTags[curFile]['acquisitionMatrix'][0], DICOMTags[curFile]['acquisitionMatrix'][2])
											acqMatrixNumCols = max(DICOMTags[curFile]['acquisitionMatrix'][1], DICOMTags[curFile]['acquisitionMatrix'][3])
											if totalNumRows % acqMatrixNumRows == 0 and totalNumCols % acqMatrixNumCols == 0:
												mosaicRows = totalNumRows / acqMatrixNumRows
												mosaicCols = totalNumCols / acqMatrixNumCols
											numRows = acqMatrixNumRows
											numCols = acqMatrixNumCols

									else:
										mat = re.match("FoV (\d+)\*(\d+)", self.__DICOMTags[curFile]['FOV'])

										if int(mat.group(1)) % phaseFOV == 0 and int(mat.group(2)) % readoutFOV == 0:
											mosaicRows = int(mat.group(1)) / phaseFOV
											mosaicCols = int(mat.group(2)) / readoutFOV
											
											# this happens if we have a dummy image
											if mosaicRows == 0:
												mosaicRows = 1
											if mosaicCols == 0:
												mosaicCols = 1

											numRows = totalNumRows / mosaicRows
											numCols = totalNumCols / mosaicCols
										else:
											# something has not worked
											# print a warning message and assume that there is only one mosaic row and one mosaic column
											
											mosaicRows = int(round(float(mat.group(1)) / float(phaseFOV)))
											mosaicCols = int(round(float(mat.group(2)) / float(readoutFOV)))
											
											numBytes = numRows * numCols * mosaicRows * mosaicCols * self.__DICOMTags[curFile]['bitsAllocated'] / 8 * self.__DICOMTags[curFile]['samplesPerPixel']
											#print "num bytes: " + str(numBytes) + ", image size: " + str(imageSize)
											# if the number of bytes taken by the rounded mosaic is equivalent to the actual image size then we are in business
											if numBytes != imageSize:
												# something has not worked
												# print a warning message and assume that there is only one mosaic row and one mosaic column
												mosaicRows = 1
												mosaicCols = 1
											
											numBytes = numRows * numCols * mosaicRows * mosaicCols * self.__DICOMTags[curFile]['bitsAllocated'] / 8 * self.__DICOMTags[curFile]['samplesPerPixel']
											if numBytes != imageSize:
												print "something is wrong with the mosaic " + curFile
												numRows = totalNumRows
												numCols = totalNumCols
											else:
												numRows = totalNumRows / mosaicRows
												numCols = totalNumCols / mosaicCols

									#if self.__DICOMTags[curFile]['FOV'] == None:
									#print "mosaicRows: " + str(mosaicRows) + ", mosaicCols: " + str(mosaicCols) + ", numRows: " + str(numRows) + ", numCols: " + str(numCols) + " " + curFile

									if self.__DICOMTags[curFile]['numSlices'] == None:
										numSlices = countedSlices 
									else:
										numSlices = self.__DICOMTags[curFile]['numSlices']
										#hexMatch = re.match('^0x[0-9a-fA-F]+$', self.__DICOMTags[curFile]['numSlices'])
										#decMatch = re.match('^\d+$', self.__DICOMTags[curFile]['numSlices'])
										#if hexMatch != None:
										#	numSlices = int(self.__DICOMTags[curFile]['numSlices'], 16)
										#elif decMatch != None:
										#	numSlices = int(self.__DICOMTags[curFile]['numSlices'])
										#else:
										#	print "Dont know how to convert numSlices: " + self.__DICOMTags[curFile]['numSlices'] + " in " + curFile
										#$if type(self.__DICOMTags[curFile]['numSlices']) is int:
										#else:
										#	numSlices = countedSlices
										#print "loading"
										#print numSlices
										#print type(numSlices)
										#print curFile
								else:
									mosaicFormat = False
									numSlices = numImages
									numVolumes = 1
									numRows = totalNumRows
									numCols = totalNumCols
								#if self.__DICOMTags[curFile]['imageType'].find("MOSAIC") != -1 and self.__DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':

								# SIEMENS old format, not mosaic but potentially multi-volume
								# so the number of slices is "countedSlices" that we computed earlier
								if mosaicFormat == False and self.__DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':
									fp = file(curFile, 'r')
									countedSlices = 0
									for curLine in fp.xreadlines():
										curLine = curLine.strip()
										if re.match("sSliceArray.asSlice\[\d+\].dReadoutFOV", curLine) != None:
											countedSlices = countedSlices + 1
										#if readFOV != None and phaseFOV != None:
										#	break
									#OV = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0051", "0x100c", curFile)
									fp.close()

									numSlices = countedSlices
									if numSlices == 0:
										numVolumes = 1
										numSlices = numImages
										#print curFile + " " + str(self.__DICOMTags[curFile]['seriesNumber']) + " " + str(numImages)
									else:
										numVolumes = numImages / numSlices
									# compute the time spacing by getting the acquisition time of the same slice in the next volume
									# non-mosaic format, the time spacing is the difference between the same slice in the next volume
									secondTimePointVolume = numSlices + 1

								if DimensionlessData == False:
									#pixelSpacing = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0028", "0x0030", curFile)
									if self.__DICOMTags[curFile]['pixelSpacing'] == None:
										print curFile + ": pixelSpacing missing"
										xSpacing = 1
										ySpacing = 1
									else:
										xSpacing = self.__DICOMTags[curFile]['pixelSpacing'][0]
										ySpacing = self.__DICOMTags[curFile]['pixelSpacing'][1]
										#(xSpacing, ySpacing) = self.__DICOMTags[curFile]['pixelSpacing'].split('\\')
									#zSpacing = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0018", "0x0050", curFile)
										
								# find out the modality and then for each modality check the particular fields to find out the dimensions and datatype of the volumes
							#if curImageIdx == 1:
							if len(self.__subjects[curSubject][curAcquisitionDate][curSeries][curImageType]) == 1 or curImageIdx == 2:#self.__DICOMTags[curFile]['imageNumber'] == 2:
								
								# LOOK FOR MOSAIC images
								#if self.__DICOMTags[curFile]['acquisitionModality'] == 'MR':
								#	pass
									#print "1"
								#elif self.__DICOMTags[curFile]['acquisitionModality'] == 'CT':
									#print self.__DICOMTags[curFile]['seriesDescription']
								#	if self.__DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':
								#		if mosaicFormat == True:
								#			print "Siemens mosaic for CT data not recognised"
										# we have all the info we need now

								#	elif self.__DICOMTags[curFile]['scannerManufacturer'] == 'GE':
								#		print "4"
								# put in the MR or CT specific fields into the episode key
									
								# this needs to go in the acqusition field since the episode can incorporate
								if self.__DICOMTags[curFile]['acquisitionModality'] == 'MR':
									
									if self.__DICOMTags[curFile]['TR'] == None:
										TR = ""
									else:
										TR = float(self.__DICOMTags[curFile]['TR'])

									if self.__DICOMTags[curFile]['TE'] == None:
										TE = ""
									else:
										TE = float(self.__DICOMTags[curFile]['TE'])

									if self.__DICOMTags[curFile]['MRFieldStrength'] == None:
										MRFieldStrength = ""
									else:
										if self.__DICOMTags[curFile]['scannerManufacturer'] == "GE MEDICAL SYSTEMS":
											MRFieldStrength = float(self.__DICOMTags[curFile]['MRFieldStrength']) / 10000
										else:
											MRFieldStrength = float(self.__DICOMTags[curFile]['MRFieldStrength'])

									if self.__DICOMTags[curFile]['flipAngle'] == None:
										flipAngle = ""
									else:
										flipAngle = float(self.__DICOMTags[curFile]['flipAngle'])
									
									if self.__DICOMTags[curFile]['numAverages'] == None:
										numAverages = ""
									else:
										numAverages = float(self.__DICOMTags[curFile]['numAverages'])
									
									curDataResource.setAcquisitionField("TR", str(TR))
									curDataResource.setAcquisitionField("TE", str(TE))
									curDataResource.setAcquisitionField("fieldStrength", str(MRFieldStrength))
									curDataResource.setAcquisitionField("flipAngle", str(flipAngle))
									curDataResource.setAcquisitionField("numAverages", str(numAverages))
									curDataResource.setAcquisitionField("sliceOrdering", str(sliceOrdering))
									curDataResource.setAcquisitionField("acquisitionTime", self.__DICOMTags[curFile]['seriesTime'])

								#elif self.__DICOMTags[curFile]['acquisitionModality'] == 'CT':
								# now put the data in 
								if DimensionlessData == True:
									dataRecordType = "binaryDataResource_t"
								else:
									dataRecordType = "mappedbinaryDataResource_t"

								if self.__DICOMTags[curFile]['bitsAllocated'] == 8:
									dataType = "uint8"
								elif self.__DICOMTags[curFile]['bitsAllocated'] == 16:
									dataType = "int16"
								else:
									dataType = 'unknown'

								curDataResource.setElementType(elementType = dataType)
								curDataResource.setByteOrder(byteOrder = 'lsbfirst')

								if DimensionlessData == False:
									curDataResource.addDimension(DataTransferProject.DataResourceDimension(\
										label = 'x', \
										size = numCols, \
										spacing = xSpacing, \
										gap = float(0), \
										units = 'mm', \
										direction = sliceOrientationX))
									
									curDataResource.addDimension(DataTransferProject.DataResourceDimension(\
										label = 'y', \
										size = numRows, \
										spacing = ySpacing, \
										gap = float(0), \
										units = 'mm', \
										direction = sliceOrientationY))
									
									#print curImgeType
									if len(self.__subjects[curSubject][curAcquisitionDate][curSeries][curImageType]) > 1:
										# this executes only on the second slice, computes the slice spacing
										secondSlicePosition = self.__DICOMTags[curFile]['slicePosition']#$getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0020", "0x0032", curFile)
										if secondSlicePosition != None:
											#firstSlicePosition = firstSlicePosition.split('\\')
											#secondSlicePosition = secondSlicePosition.split('\\')
											#print curFile
											#print " ".join(firstSlicePosition)
											#print " ".join(secondSlicePosition)
											xx = float(firstSlicePosition[0]) - float(secondSlicePosition[0])
											yy = float(firstSlicePosition[1]) - float(secondSlicePosition[1])
											zz = float(firstSlicePosition[2]) - float(secondSlicePosition[2])

											zSpacing = math.sqrt(xx * xx + yy * yy + zz * zz)
											# if we get a slice thickness of zero, try the spacing between slices key
											if zSpacing == 0.0:
												zSpacing = self.__DICOMTags[curFile]['spacingBetweenSlices']
												if zSpacing == None:
													# then try slice thickness key
													zSpacing = self.__DICOMTags[curFile]['sliceThickness']#getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0018", "0x0050", curFile)
													if zSpacing == None:
														print curFile + ": zSpacing really missing"
														zSpacing = 1
													else:
														zSpacing = float(zSpacing)		
												else:
													zSpacing = float(zSpacing)		
										else:
											print curFile + ": zSpacing really missing"
											zSpacing = 1
									else:
										print curFile + ": zSpacing really missing"
										zSpacing = 1
									
									zDimension = DataTransferProject.DataResourceDimension(\
											label = 'z', \
											size = numSlices, \
											spacing = zSpacing, \
											gap = float(0), \
											units = 'mm', \
											direction = sliceOrientationZ)
	
									if mosaicFormat == True:
										zDimension.setMosaicSize(mosaicRows)

									#if mosaicFormat == True:
								#if DimensionlessData == False:
								# calculate the directories for each subject, study, visit, episode, acqusition
							#if len(self.__subjects[curSubject][curAcquisitionDate][curSeries][curImageType]) == 1 or curImageIdx == 2:#self.__DICOMTags[curFile]['imageNumber'] == 2:
							if curImageIdx > 1 and DimensionlessData == False:
								if numVolumes > 1 and curImage == secondTimePointVolume:
									tSpacing = acquisitionTime - firstAcquisitionTime
									
									if tSpacing > 0:
										curDataResource.addDimension(zDimension)
										curDataResource.addDimension(DataTransferProject.DataResourceDimension(\
											label = 't', \
											size = numVolumes, \
											spacing = tSpacing, \
											gap = float(0), \
											units = 's', \
											datapoints = list(numpy.arange(0, tSpacing * (numVolumes - 1), tSpacing))))
										del zDimension
									else:
										if mosaicFormat == False:
											# we made a mistake and the number of volumes is actually the number of slices
											zDimension.setSize(numVolumes)
											curDataResource.addDimension(zDimension)
										else:
											# mosaic format, something is wrong with the image header, so we actually have a timeseries volume
											# get the tSpacing from the TR if MR, otherwise just use 1
											if DICOMTags[curFile]['acquisitionModality'] == 'MR':
												tSpacing = DICOMTags[curFile]['TR'] / 1000
											else:
												tSpacing = 1
											curDataResource.addDimension(DataTransferProject.DataResourceDimension(\
											label = 't', \
											size = numVolumes, \
											spacing = tSpacing, \
											gap = float(0), \
											units = 's', \
											datapoints = list(numpy.arange(0, tSpacing * (numVolumes - 1), tSpacing))))
	
								#if numVolumes > 1 and curImage == secondTimePointVolume:
							#if curImageIdx > 1 and DimensionlessData == False:
							# mosaic: the entire volume is acquired in the one DICOM file, the second time point we need is in the second file
							# non-mosaic: the second time point is the acquisition time of the same slice in the second volume
							# check for MR - DTI - SIEMENS data
							if self.__DICOMTags[curFile]['acquisitionModality'] == 'MR':
								if self.__DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':
									#details = getDICOMKeyDCKEY.getDICOMMultipleKeyDCKEY(["0x0019,0x100c", "0x0019,0x100e"], curFile)
									if self.__DICOMTags[curFile]['BValue'] != None:
										BValues.append(int(self.__DICOMTags[curFile]['BValue']))
										# correct BVector for slice orientation
										# X component is X slice direction dot BVector
										# Y component is Y slice direction dot BVector
										# Z component is Z slice direction dot BVector
										if self.__DICOMTags[curFile]['BVector'] != None and int(self.__DICOMTags[curFile]['BValue']) > 0:
											#curBVector = self.__DICOMTags[curFile]['BVector'].split(',')
											curBVector = self.__DICOMTags[curFile]['BVector']
											for idx in range(len(curBVector)):
												curBVector[idx] = float(curBVector[idx])
											curBVectorMag = 0.0
											for idx in range(len(curBVector)):
												curBVectorMag = curBVectorMag + curBVector[idx] * curBVector[idx]
											curBVectorMag = math.sqrt(curBVectorMag)
											for idx in range(len(curBVector)):
												curBVector[idx] = curBVector[idx] / curBVectorMag
											# this is for FSL, may transfer this to dt_checkout_volume
											correctedBVector = curBVector
											#correctedBVector[0] = curBVector[0] * sliceOrientationX[0] + curBVector[1] * sliceOrientationX[1] + curBVector[2] * sliceOrientationX[2]
											#correctedBVector[1] = curBVector[0] * sliceOrientationY[0] + curBVector[1] * sliceOrientationY[1] + curBVector[2] * sliceOrientationY[2]
											#correctedBVector[1] = -correctedBVector[1] # to be compatible with dcm2nii, haven't figure this one out yet
											#correctedBVector[2] = curBVector[0] * sliceOrientationZ[0] + curBVector[1] * sliceOrientationZ[1] + curBVector[2] * sliceOrientationZ[2]
											BVectors.append(correctedBVector)
											DTIImage = True
										else:
											BVectors.append([float(0), float(0), float(0)])
									elif self.__DICOMTags[curFile]['CSAImageHeader'] != None:
										# if this isnt a mosaic file, we only need to get the diffusion directions from the first slice in each volume
										if mosaicFormat == True or (curImageIdx % numSlices) == 1:
												# check for the diffusion information in the CSA Image Header
												if self.__DICOMTags[curFile]['CSAHeaderFormat'] != None and self.__DICOMTags[curFile]['CSAHeaderFormat'] == 'SIEMENS CSA HEADER':
													BVectorAndBValue = getBValueBVectorSiemensCSA(self.__DICOMTags[curFile]['CSAImageHeader'])
													if BVectorAndBValue == None:
														print curFile
												else:
													BVectorAndBValue = {}
													BVectorAndBValue['BValue'] = 0
													BVectorAndBValue['BVector'] = [0, 0, 0]
												BValues.append(BVectorAndBValue['BValue'])
												BVectors.append(BVectorAndBValue['BVector'])
												if BVectorAndBValue['BValue'] != 0:
													DTIImage = True
													#print "Got DTI Data via CSA image header: " + str(BVectorAndBValue['BValue']) + ", [" + ", ".join(["%s" % (v) for v in BVectorAndBValue['BVector']]) + "]"
									else:
										BValues.append(0)
										BVectors.append([0, 0, 0])
							if curImageIdx == 1:#self.__DICOMTags[curFile]['imageNumber'] == 1:
								firstImageSize = imageSize
								firstImageOffset = imageOffset
								firstFileName = curFile
								firstAcquisitionTime = acquisitionTime

							curImageIdx = curImageIdx + 1
						#for curImage in sorted(self.__subjects[curSubject][curAcquisitionDate][curDirectory][curSeries].iterkeys()):
						# add in the BVectors and BValues if we have found a DTI volume
						if DTIImage == True:
							curDataResource.setAcquisitionField(key = 'BValues', value = ",".join(["%s" % (v) for v in BValues]))
							for idx in range(len(BVectors)):
								BVectors[idx] = ",".join(["%s" % (v) for v in BVectors[idx]])
							curDataResource.setAcquisitionField(key = "BVectors", value = ";".join(BVectors))
						curDataResource.setFileListSizesAndOffsets(fileList = fileList, sizes = sizes, offsets = offsets)
						curDataResource.createSubjectObject(name = self.__DICOMTags[curFile]['subjectName'], address = self.__DICOMTags[curFile]['subjectAddress'], DOB = self.__DICOMTags[curFile]['subjectDOB'], sex = self.__DICOMTags[curFile]['subjectSex'])
						dataResources.append(curDataResource)
						#print curDataResource
					#for curImageType in subjects[curSubject][curAcquisitionDate][curSeries].iterkeys())
				#for curSeries in subjects[curSubject][curAcquisitionDate].iterkeys():
			#for curAcquisitionDate in subjects[curSubject].iterkeys():
		#for curSubject in subjects.iterkeys():
		return dataResources
	#def makeDataResources(self, subjectHashIDMapping = None):
#class DICOMFileReader(FileReader):

def getBValueBVectorSiemensCSA(CSAImageHeader):	

	CSAEntries = ['EchoLinePosition',
			'EchoColumnPosition',
			'EchoPartitionPosition',
			'UsedChannelMask',
			'Actual3DImaPartNumber',
			'ICE_Dims',
			'B_value',
			'Filter1',
			'Filter2',
			'ProtocolSliceNumber',
			'RealDwellTime',
			'PixelFile',
			'PixelFileName',
			'SliceMeasurementDuration',
			'SequenceMask',
			'AcquisitionMatrixText',
			'MeasuredFourierLines',
			'FlowEncodingDirection',
			'FlowVenc',
			'PhaseEncodingDirectionPositive',
			'NumberOfImagesInMosaic',
			'DiffusionGradientDirection',
			'ImageGroup',
			'SliceNormalVector',
			'DiffusionDirectionality',
			'TimeAfterStart',
			'FlipAngle',
			'SequenceName',
			'RepetitionTime',
			'EchoTime',
			'NumberOfAverages',
			'VoxelThickness',
			'VoxelPhaseFOV',
			'VoxelReadoutFOV',
			'VoxelPositionSag',
			'VoxelPositionCor',
			'VoxelPositionTra',
			'VoxelNormalSag',
			'VoxelNormalCor',
			'VoxelNormalTra',
			'VoxelInPlaneRot',
			'ImagePositionPatient',
			'ImageOrientationPatient',
			'PixelSpacing',
			'SliceLocation',
			'SliceThickness',
			'SpectrumTextRegionLabel',
			'Comp_Algorithm',
			'Comp_Blended',
			'Comp_ManualAdjusted',
			'Comp_AutoParam',
			'Comp_AdjustedParam',
			'Comp_JobID',
			'FMRIStimulInfo',
			'FlowEncodingDirectionString',
			'RepetitionTimeEffective']

	fp = StringIO(CSAImageHeader)
	# tells us what format the CSA Header is in
	magicNumber = fp.read(4)
	if magicNumber == "SV10":
		# format 1
		unused = fp.read(4)
		numElements = unpack('I', fp.read(4))[0]
	else:
		numElements = unpack('I', fp.read(4))[0]
		unused = fp.read(4)

	if numElements < 0 or numElements > 128:
		print "numElements should be between 0 and 128 inclusive, returning blank"
		return None
	unused = unpack('I', fp.read(4))[0]
	if unused != 77:
		print "unused is not 77"
	if numElements > len(CSAEntries):
		numElements = len(CSAEntries)
	elements = []
	for curElement in range(numElements):

		curElementHash = {}
		curElementHash['name'] = fp.read(64).strip(" \x00")

		if curElementHash['name'].find(CSAEntries[curElement]) == 0:
			curElementHash['name'] = CSAEntries[curElement]
		else:
			print "name doesnt match dictionary in CSA Header"

		curElementHash['VM'] = unpack('I', fp.read(4))[0]
		curElementHash['VR'] = fp.read(4).strip()
		curElementHash['SyngoDT'] = unpack('I', fp.read(4))[0]
		curElementHash['numItems'] = unpack('I', fp.read(4))[0]
		curElementHash['XX'] = unpack('I', fp.read(4))[0]
		curElementHash['items'] = []
		for curItem in range(curElementHash['numItems']):
			curItemHash = {}
			curItemHash['XX'] = []
			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
			if magicNumber == "SV10":
				curLength = curItemHash['XX'][1]
			else:
				curLength = curItemHash['XX'][0] - curElementHash['numItems']

			curItemHash['val'] = fp.read(curLength).strip(" \x00")
			#i = curItemHash['val'].index('\x00')
			#curItemHash['val'] = curItemHash['val'][:i]
			# read some bytes to pad, since the the elements should be on 4-byte boundaries
			unused = fp.read((4 - (curLength % 4)) % 4)
			curElementHash['items'].append(curItemHash)
		#for curItem in range(numItems):
		elements.append(curElementHash)
	#for curElement in range(numElements):
	BStats = {}
	BStats['BVector'] = [0, 0, 0]
	BStats['BValue'] = 0
	for curElement in range(len(elements)):
		#print elements[curElement]['name']
		if elements[curElement]['name'] == 'DiffusionGradientDirection':
			if elements[curElement]['numItems'] == 6:
				curBVector = []
				curBVector.append(float(elements[curElement]['items'][0]['val'].strip()))
				curBVector.append(float(elements[curElement]['items'][1]['val'].strip()))
				curBVector.append(float(elements[curElement]['items'][2]['val'].strip()))
			else:
				#print str(elements[curElement]['numItems'])
				curBVector = [0, 0, 0]
			BStats['BVector'] = curBVector

		if elements[curElement]['name'] == 'B_value':
			if elements[curElement]['numItems'] > 0:
				BStats['BValue'] = int(elements[curElement]['items'][0]['val'].strip())
			else:
				#print "numItems == 0"
				BStats['BValue'] = 0
	#print BStats['BValue']
	#print BStats['BVector']
	return BStats


#def getBValueBVectorSiemensCSA(CSAImageHeader):	
#
#	CSAEntries = ['EchoLinePosition',
#			'EchoColumnPosition',
#			'EchoPartitionPosition',
#			'UsedChannelMask',
#			'Actual3DImaPartNumber',
#			'ICE_Dims',
#			'B_value',
#			'Filter1',
#			'Filter2',
#			'ProtocolSliceNumber',
#			'RealDwellTime',
#			'PixelFile',
#			'PixelFileName',
#			'SliceMeasurementDuration',
#			'SequenceMask',
#			'AcquisitionMatrixText',
#			'MeasuredFourierLines',
#			'FlowEncodingDirection',
#			'FlowVenc',
#			'PhaseEncodingDirectionPositive',
#			'NumberOfImagesInMosaic',
#			'DiffusionGradientDirection',
#			'ImageGroup',
#			'SliceNormalVector',
#			'DiffusionDirectionality',
#			'TimeAfterStart',
#			'FlipAngle',
#			'SequenceName',
#			'RepetitionTime',
#			'EchoTime',
#			'NumberOfAverages',
#			'VoxelThickness',
#			'VoxelPhaseFOV',
#			'VoxelReadoutFOV',
#			'VoxelPositionSag',
#			'VoxelPositionCor',
#			'VoxelPositionTra',
#			'VoxelNormalSag',
#			'VoxelNormalCor',
#			'VoxelNormalTra',
#			'VoxelInPlaneRot',
#			'ImagePositionPatient',
#			'ImageOrientationPatient',
#			'PixelSpacing',
#			'SliceLocation',
#			'SliceThickness',
#			'SpectrumTextRegionLabel',
#			'Comp_Algorithm',
#			'Comp_Blended',
#			'Comp_ManualAdjusted',
#			'Comp_AutoParam',
#			'Comp_AdjustedParam',
#			'Comp_JobID',
#			'FMRIStimulInfo',
#			'FlowEncodingDirectionString',
#			'RepetitionTimeEffective']
#
#
#	fp = StringIO(CSAImageHeader)
#	# tells us what format the CSA Header is in
#	magicNumber = fp.read(4)
#	if magicNumber == "SV10":
#		# format 1
#		unused = fp.read(4)
#		numElements = unpack('I', fp.read(4))[0]
#	else:
#		numElements = unpack('I', fp.read(4))[0]
#		unused = fp.read(4)
#
#	if numElements < 0 or numElements > 128:
#		print "numElements should be between 0 and 128 inclusive, returning blank"
#		return None
#	unused = unpack('I', fp.read(4))[0]
#	if unused != 77:
#		print "unused is not 77"
#
#	elements = []
#	for curElement in range(numElements):
#
#		curElementHash = {}
#		curElementHash['name'] = fp.read(64).strip(" \x00")
#		if curElementHash['name'].find(CSAEntries[curElement]) == 0:
#			curElementHash['name'] = CSAEntries[curElement]
#		else:
#			print "name doesnt match dictionary in CSA Header"
#
#		curElementHash['VM'] = unpack('I', fp.read(4))[0]
#		curElementHash['VR'] = fp.read(4).strip()
#		curElementHash['SyngoDT'] = unpack('I', fp.read(4))[0]
#		curElementHash['numItems'] = unpack('I', fp.read(4))[0]
#		curElementHash['XX'] = unpack('I', fp.read(4))[0]
#		curElementHash['items'] = []
#		for curItem in range(curElementHash['numItems']):
#			curItemHash = {}
#			curItemHash['XX'] = []
#			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
#			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
#			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
#			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
#			if magicNumber == "SV10":
#				curLength = curItemHash['XX'][1]
#			else:
#				curLength = curItemHash['XX'][0] - curElementHash['numItems']
#
#			curItemHash['val'] = fp.read(curLength).strip(" \x00")
#			#i = curItemHash['val'].index('\x00')
#			#curItemHash['val'] = curItemHash['val'][:i]
#			# read some bytes to pad, since the the elements should be on 4-byte boundaries
#			unused = fp.read((4 - (curLength % 4)) % 4)
#			curElementHash['items'].append(curItemHash)
#		#for curItem in range(numItems):
#		elements.append(curElementHash)
#	#for curElement in range(numElements):
#	BStats = {}
#	BStats['BVector'] = [0, 0, 0]
#	BStats['BValue'] = 0
#	for curElement in range(len(elements)):
#		#print elements[curElement]['name']
#		if elements[curElement]['name'] == 'DiffusionGradientDirection':
#			if elements[curElement]['numItems'] == 6:
#				curBVector = []
#				curBVector.append(float(elements[curElement]['items'][0]['val'].strip()))
#				curBVector.append(float(elements[curElement]['items'][1]['val'].strip()))
#				curBVector.append(float(elements[curElement]['items'][2]['val'].strip()))
#			else:
#				#print str(elements[curElement]['numItems'])
#				curBVector = [0, 0, 0]
#			BStats['BVector'] = curBVector
#
#		if elements[curElement]['name'] == 'B_value':
#			if elements[curElement]['numItems'] > 0:
#				BStats['BValue'] = int(elements[curElement]['items'][0]['val'].strip())
#			else:
#				#print "numItems == 0"
#				BStats['BValue'] = 0
#	#print BStats['BValue']
#	#print BStats['BVector']
#	return BStats
#
