from lxml import etree

import os

userFieldFormats = ['integer', 'floating', 'boolean', 'date', 'string']
userFieldLevels = ['subjectInfo', 'visit', 'study', 'episode', 'acquisition']

# reserved fields are those that are either part of the XCEDE standard or added automatically by data importers
reservedFieldsXCEDE = {	'subjectInfo': [('birthdate', 'date'), ('sex', 'string'), ('species', 'string')],
						'visit': [('visitInfo', 'string')],
						'study': [('studyInfo', 'string')],
						'episode': [('episodeInfo', 'string')],
						'acquisition': [('acquisitionInfo', 'string'), ('dataResourceRef', 'string'), ('dataRef', 'string')] }

# reserved fields that are added by the data importing programs, these are not to be fiddled with
reservedFieldsUser = { 	'subjectInfo': [('hash', 'string')],
						'visit': [('subjectAgeAtVisit', 'integer')],
						'study': [],
						'episode': [],
						'acquisition': [('TR', 'floating'), ('TE', 'floating'), ('fieldStrength', 'floating'), ('flipAngle', 'floating'), ('numAverages', 'integer'), ('scannerManufacturer', 'string'), ('acquisitionTime', 'floating')] } # MR fields
# CT fields

def isFieldReserved(level = None, name = None):
	if level in reservedFieldsXCEDE:
		for curField in reservedFieldsXCEDE[level]:
			if name == curField[0]:
				return True

	if level in reservedFieldsUser:
		for curField in reservedFieldsUser[level]:
			if name == curField[0]:
				return True
	
	return False

def getReservedFieldFormat(level = None, name = None):
	if level in reservedFieldsXCEDE:
		for curField in reservedFieldsXCEDE[level]:
			if name == curField[0]:
				return curField[1]

	if level in reservedFieldsUser:
		for curField in reservedFieldsUser[level]:
			if name == curField[0]:
				return curField[1]
	return None

""" Contains the user specified fields, so the ones not in the XCEDE spec, that can be added to the database """
""" This class reads the file that contains the specs, both the project and database-wide files """
""" The database-wide user field file is <database directory>/userfields.txt """
""" The user field file is <database directory>/<project directory>/userfields.txt """

""" File format: each field has its attributes on a single line and the attributes are tab delimited """
""" The attributes are: level, name, format, default value """
""" level: indicates the level of the hierarchy that the field resides, this can be """
""" 	subjectInfo, study, visit, episode, acquisition """
""" name: the tag of the field """
""" format: the format of the text in the field, can be: numeric, boolean, string, date """
""" default value: the value to use when creating the field """
""" note that "please edit" is a valid value for any field, since most of these fields will need to be manually entered """

class UserFields:

	def __init__(self, projectName=None, databaseDirectory=None):
		self.projectName = projectName
		self.databaseDirectory = databaseDirectory
		# these are the database-wide fields
		self.databaseFields = self.readFieldFile(os.path.join(databaseDirectory, 'userfields.txt'))
		# these are the project-specific fields
		if self.projectName != None:
			self.projectFields = self.readFieldFile(os.path.join(databaseDirectory, projectName, 'userfields.txt'))
		else:
			self.projectFields = []

	def saveProjectXML(self):
		self.projectXMLRoot.write(os.path.join(self.databaseDirectory, self.projectName, 'projects.xml'), pretty_print=True)

	def saveDatabaseFieldFile(self):
		fp = file(os.path.join(self.databaseDirectory, 'userfields.txt'), 'w')
		for curField in self.databaseFields:
			fp.write(curField['level'] + "\t" + curField['name'] + "\t" + curField['format'] + "\t" + curField['defaultValue'] + "\n")
		fp.close()
	
	def saveProjectFieldFile(self):
		fp = file(os.path.join(self.databaseDirectory, self.projectName, 'userfields.txt'), 'w')
		for curField in self.projectFields:
			fp.write(curField['level'] + "\t" + curField['name'] + "\t" + curField['format'] + "\t" + curField['defaultValue'] + "\n")
		fp.close()

	def appendProjectField(self, level=None, name=None, format=None, defaultValue=None):
		curFieldHash = {}
		curFieldHash['level'] = level
		curFieldHash['name'] = name
		curFieldHash['format'] = format
		curFieldHash['defaultValue'] = defaultValue
		self.projectFields.append(curFieldHash)

	def appendDatabaseField(self, level=None, name=None, format=None, defaultValue=None):
		curFieldHash = {}
		curFieldHash['level'] = level
		curFieldHash['name'] = name
		curFieldHash['format'] = format
		curFieldHash['defaultValue'] = defaultValue
		self.databaseFields.append(curFieldHash)
	
	""" Returns an array of hash tables, each hash table contains the fields for one external field: level, name, format, defaultValue """
	def readFieldFile(self, fileName=None):
		
		fieldArray = []
		
		if os.path.isfile(fileName) == True:
			fp = file(fileName, 'r')
			for curLine in fp.xreadlines():
				curLine = curLine.strip()
				curLineArray = curLine.split("\t")
				curFieldHash = {}
				curFieldHash['level'] = curLineArray[0]
				curFieldHash['name'] = curLineArray[1]
				curFieldHash['format'] = curLineArray[2]
				if len(curLineArray) < 4:
					curFieldHash['defaultValue'] = ''
				else:
					curFieldHash['defaultValue'] = curLineArray[3]

				fieldArray.append(curFieldHash)
			fp.close()

		return fieldArray

	""" Adds the fields with default values into an lxml XML root """
	def addFieldsInXML(self, projectXMLRoot=None):
		self.addFieldsFromOneSet(self.databaseFields, projectXMLRoot=projectXMLRoot)
		self.addFieldsFromOneSet(self.projectFields, projectXMLRoot=projectXMLRoot)
	
	""" Deletes a field from the XML file, actually does the dirty work """
	def deleteFieldInXML(self, curFields=None, projectXMLRoot=None):
		for curField in curFields:
			if curField['level'].find('Info') != -1:
				XPathQuery = '/XCEDE/' + curField['level'][:-4] + "/" + curField['level']
			else:
				XPathQuery = '/XCEDE/' + curField['level']
			# find all the nodes that contain the level of the current element
			curLevelNodes = projectXMLRoot.xpath(XPathQuery)
			# go through each node and verify that the field is there
			for curLevelNode in curLevelNodes:
				for curChildren in curLevelNode.iterchildren(tag=curField['name']):
					curLevelNode.remove(curChildren)
			#for curNode in curNodes:
		#for curField in curFields:
		return None


	""" Adds the fields from one set of fields into the XML file, actually does the dirty work """
	def addFieldsFromOneSet(self, curFields=None, projectXMLRoot=None):
		"""subjectInfo, study, visit, episode, acquisition """
		for curField in curFields:
			if curField['level'].find('Info') != -1:
				XPathQuery = '/XCEDE/' + curField['level'][:-4] + "/" + curField['level']
			else:
				XPathQuery = '/XCEDE/' + curField['level']
			# find all the nodes that contain the level of the current element
			curLevelNodes = projectXMLRoot.xpath(XPathQuery)
			# go through each node and verify that the field is there
			for curLevelNode in curLevelNodes:
				curFieldFound = False
				for curChildren in curLevelNode.iterchildren(tag=curField['name']):
					curFieldFound = True
				if curFieldFound == False:
					newElement = etree.SubElement(curLevelNode, curField['name'])
					newElement.text = curField['defaultValue']
			#for curNode in curNodes:
		#for curField in curFields:
		# fix up the location of the hash elements in subjectInfo, it should be at the bottom, but I originally put it at the top, my bad
		subjectInfoNodes = projectXMLRoot.xpath('/XCEDE/subject/subjectInfo')
		for curSubjectInfoNode in subjectInfoNodes:
			for hashNode in curSubjectInfoNode.iterchildren(tag="hash"):
				curHashValue = hashNode.text
				nodeToDelete = hashNode
			curSubjectInfoNode.remove(hashNode)
			# now add it in again, so that it will be at the end
			tempNode = etree.SubElement(curSubjectInfoNode, "hash")
			tempNode.text = curHashValue

		return None

	def fieldArrayToString(self, fieldArray):
		# use the recipe I got from the net
		# http://code.activestate.com/recipes/267662/
		labels = ('level', 'name', 'format', 'default value')
		data = ''
		for curField in fieldArray:
			#outputString = outputString + "\t" + curField['level'] + "\t" + curField['name'] + "\t\t" + curField['format'] + "\t\t" + curField['defaultValue'] + "\n"
			data = data + curField['level'] + "\t" + curField['name'] + "\t" + curField['format'] + "\t" + curField['defaultValue'] + "\n"
		rows = [row.strip().split("\t") for row in data.splitlines()]
		return indent([labels]+rows, hasHeader=True)

		#return outputString
	
	def __str__(self):
		outputString = ''
		if self.databaseFields != None:
			outputString = outputString + "Database-wide fields:\n"
			outputString = outputString + self.fieldArrayToString(self.databaseFields)

		if self.projectFields != None:
			outputString = outputString + "Project-specific fields:\n"
			outputString = outputString + self.fieldArrayToString(self.projectFields)
		return outputString
	
	def databaseFieldIdx(self, level=None, name=None):
		fieldIdx = -1
		for curField in range(len(self.databaseFields)):
			if self.databaseFields[curField]['level'] == level and self.databaseFields[curField]['name'] == name:
				fieldIdx = curField
				break
		return fieldIdx
	
	def projectFieldIdx(self, level=None, name=None):
		fieldIdx = -1
		for curField in range(len(self.projectFields)):
			if self.projectFields[curField]['level'] == level and self.projectFields[curField]['name'] == name:
				fieldIdx = curField
				break
		return fieldIdx
	
	def fieldFormat(self, level = None, name = None):
		curDatabaseIdx = self.databaseFieldIdx(level = level, name=name)
		curProjectIdx = self.projectFieldIdx(level = level, name=name)

		if curDatabaseIdx != -1:
			return self.databaseFields[curDatabaseIdx]['format']
		elif curProjectIdx != -1:
			return self.projectFields[curProjectIdx]['format']
		elif isFieldReserved(level = level, name = name):
			return getReservedFieldFormat(level = level, name = name)
	
	def fieldScope(self, level = None, name = None):
		if not (isinstance(level, str) and isinstance(name, str)):
			raise Exceptions.InvalidType()
		elif self.databaseFieldIdx(level = level, name = name) != -1:
			return "database"
		elif self.projectFieldIdx(level = level, name = name) != -1:
			return "project"
		elif isFieldReserved(level = level, name = name):
			return "reserved"
		else:
			return None

	def replaceField(self, level=None, name=None, format=None, defaultValue=None):
		curDatabaseIdx = self.databaseFieldIdx(self, level=name, name=name)
		curProjectIdx = self.projectFieldIdx(self, level=name, name=name)

		if curDatabaseIdx != -1:
			self.databaseFields[curDatabaseIdx]['format'] = format
			self.databaseFields[curDatabaseIdx]['defaultValue'] = defaultValue
		elif curProjectIdx != -1:
			self.projectFields[curProjectIdx]['format'] = format
			self.projectFields[curProjectIdx]['defaultValue'] = defaultValue

import cStringIO,operator

def indent(rows, hasHeader=False, headerChar='-', delim=' | ', justify='left',
           separateRows=False, prefix='', postfix='', wrapfunc=lambda x:x):
    """Indents a table by column.
       - rows: A sequence of sequences of items, one sequence per row.
       - hasHeader: True if the first row consists of the columns' names.
       - headerChar: Character to be used for the row separator line
         (if hasHeader==True or separateRows==True).
       - delim: The column delimiter.
       - justify: Determines how are data justified in their column. 
         Valid values are 'left','right' and 'center'.
       - separateRows: True if rows are to be separated by a line
         of 'headerChar's.
       - prefix: A string prepended to each printed row.
       - postfix: A string appended to each printed row.
       - wrapfunc: A function f(text) for wrapping text; each element in
         the table is first wrapped by this function."""
    # closure for breaking logical rows to physical, using wrapfunc
    def rowWrapper(row):
        newRows = [wrapfunc(item).split('\n') for item in row]
        return [[substr or '' for substr in item] for item in map(None,*newRows)]
    # break each logical row into one or more physical ones
    logicalRows = [rowWrapper(row) for row in rows]
    # columns of physical rows
    columns = map(None,*reduce(operator.add,logicalRows))
    # get the maximum of each column by the string length of its items
    maxWidths = [max([len(str(item)) for item in column]) for column in columns]
    rowSeparator = headerChar * (len(prefix) + len(postfix) + sum(maxWidths) + \
                                 len(delim)*(len(maxWidths)-1))
    # select the appropriate justify method
    justify = {'center':str.center, 'right':str.rjust, 'left':str.ljust}[justify.lower()]
    output=cStringIO.StringIO()
    if separateRows: print >> output, rowSeparator
    for physicalRows in logicalRows:
        for row in physicalRows:
            print >> output, \
                prefix \
                + delim.join([justify(str(item),width) for (item,width) in zip(row,maxWidths)]) \
                + postfix
        if separateRows or hasHeader: print >> output, rowSeparator; hasHeader=False
    return output.getvalue()

# written by Mike Brown
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/148061
def wrap_onspace(text, width):
    """
    A word-wrap function that preserves existing line breaks
    and most spaces in the text. Expects that existing line
    breaks are posix newlines (\n).
    """
    return reduce(lambda line, word, width=width: '%s%s%s' %
                  (line,
                   ' \n'[(len(line[line.rfind('\n')+1:])
                         + len(word.split('\n',1)[0]
                              ) >= width)],
                   word),
                  text.split(' ')
                 )

import re
def wrap_onspace_strict(text, width):
    """Similar to wrap_onspace, but enforces the width constraint:
       words longer than width are split."""
    wordRegex = re.compile(r'\S{'+str(width)+r',}')
    return wrap_onspace(wordRegex.sub(lambda m: wrap_always(m.group(),width),text),width)

import math
def wrap_always(text, width):
    """A simple word-wrap function that wraps text on exactly width characters.
       It doesn't split the text in words."""
    return '\n'.join([ text[width*i:width*(i+1)] \
                       for i in xrange(int(math.ceil(1.*len(text)/width))) ])
    
if __name__ == '__main__':
    labels = ('First Name', 'Last Name', 'Age', 'Position')
    data = \
    '''John,Smith,24,Software Engineer
       Mary,Brohowski,23,Sales Manager
       Aristidis,Papageorgopoulos,28,Senior Reseacher'''
    rows = [row.strip().split(',')  for row in data.splitlines()]

    print 'Without wrapping function\n'
    print indent([labels]+rows, hasHeader=True)
    # test indent with different wrapping functions
    width = 10
    for wrapper in (wrap_always,wrap_onspace,wrap_onspace_strict):
        print 'Wrapping function: %s(x,width=%d)\n' % (wrapper.__name__,width)
        print indent([labels]+rows, hasHeader=True, separateRows=True,
                     prefix='| ', postfix=' |',
                     wrapfunc=lambda x: wrapper(x,width))
    
    # output:
    #
    #Without wrapping function
    #
    #First Name | Last Name        | Age | Position         
    #-------------------------------------------------------
    #John       | Smith            | 24  | Software Engineer
    #Mary       | Brohowski        | 23  | Sales Manager    
    #Aristidis  | Papageorgopoulos | 28  | Senior Reseacher 
    #
    #Wrapping function: wrap_always(x,width=10)
    #
    #----------------------------------------------
    #| First Name | Last Name  | Age | Position   |
    #----------------------------------------------
    #| John       | Smith      | 24  | Software E |
    #|            |            |     | ngineer    |
    #----------------------------------------------
    #| Mary       | Brohowski  | 23  | Sales Mana |
    #|            |            |     | ger        |
    #----------------------------------------------
    #| Aristidis  | Papageorgo | 28  | Senior Res |
    #|            | poulos     |     | eacher     |
    #----------------------------------------------
    #
    #Wrapping function: wrap_onspace(x,width=10)
    #
    #---------------------------------------------------
    #| First Name | Last Name        | Age | Position  |
    #---------------------------------------------------
    #| John       | Smith            | 24  | Software  |
    #|            |                  |     | Engineer  |
    #---------------------------------------------------
    #| Mary       | Brohowski        | 23  | Sales     |
    #|            |                  |     | Manager   |
    #---------------------------------------------------
    #| Aristidis  | Papageorgopoulos | 28  | Senior    |
    #|            |                  |     | Reseacher |
    #---------------------------------------------------
    #
    #Wrapping function: wrap_onspace_strict(x,width=10)
    #
    #---------------------------------------------
    #| First Name | Last Name  | Age | Position  |
    #---------------------------------------------
    #| John       | Smith      | 24  | Software  |
    #|            |            |     | Engineer  |
    #---------------------------------------------
    #| Mary       | Brohowski  | 23  | Sales     |
    #|            |            |     | Manager   |
    #---------------------------------------------
    #| Aristidis  | Papageorgo | 28  | Senior    |
    #|            | poulos     |     | Reseacher |
    #---------------------------------------------
