#!/usr/bin/python

from lxml import etree
import sys
import os
import errno
import getopt
import DataTransferConstants
import DataTransferPatterns
import re
import shutil
import time
import fnmatch

def usage():
	print "Usage: " + sys.argv[0] + " [--database-directory=<>] [-h] --output-directory=<> --output-format=<> --output-pattern=<> --dti-output-format=<fsl or mrtrix> --sort-output=sort1,sort2... --match-input-file="

	print ""
	print "--database-directory: the root directory of the database"
	print "-h: prints this help message, program exits"
	print "--output-directory: the directory to place the output files"
	print "--output-format: must be NIFTI or MINC"
	print "--output-pattern: each file will be named according to this pattern. Special sequences of characters can be used to substitute IDs into the filenames as follows:"
	print ""
	print "%p: project name"
	print "%t: subject type"
	print "%s: subject ID"
	print "%v: visit ID (date)"
	print "%e: episode ID"
	print "%a: acquisition ID"
	print "%d: today's date (YYYYMMDD)"
	print "%n: the index of the file in the current set of checked out files, so the first file will be 1, second this will be 2, and so on"
	print "%%: the character %"
	print ""
	print "Input field patterns:"
	print ""
	print "String fields (study-pattern, episode-pattern) permits the use of wildcard patterns in the same way as used in the shell for file matching"
	print ""
	print "Numeric patterns (subject-pattern, visit-pattern, acquisition-pattern) uses MATLAB-like syntax"
	print "\t--subject-pattern=lowerLimit:increment:upperLimit, lowerLimit:*, *:upperlimit"
	print "\t\t so --subject-pattern=1:5:21 will select subjects with IDs 1, 6, 11, 16, 21"
	print "\t\t so --subject-pattern=*:21 will select subjects with IDs below and including 21"
	print "\t\t so --subject-pattern=21:* will select subjects with IDs with 21 or above"
	print ""
	print "IMPORTANT:"
	print "\tThe wildcard character, *, is interpreted by the shell prior to program execution and thus may return a list of files in the current directory"
	print "\tTo avoid this, escape any wildcards with two backslashes, \\\\*, so that bash does not perform file matching"
	print ""
	print "--database-directory NOTES"
	print "\tThe project directory is the root directory of the database and it is an optional command line argument."
	print "\tIf the project directory given on the command line then that value will be used."
	print "\tIf it is not specified on the command line it will search the environment variable DT_PROJECT_DIR"
	print "\tIf the environment variable is empty, the value in DataTransferConstants.databaseDirectory will be used: " + DataTransferConstants.databaseDirectory
	print
	print "--dti-output-pattern=fsl (default) will produce a bval and bvecs file with each DTI volume found"
	print "--dti-output-pattern=mrtrix will produce a grad file, compatible with mrtrix, with each DTI volume found"
	print
	print "--sort-output=key1,key2,key3... is used to place output files in subdirectories of the output directory (--output-directory)"
	print " refer to the substitution rules for --output-pattern for how to insert the meta-data into the directory names"
	print " the extra key %, is used here to insert a comma into the directory name"
	print " This feature is intended to be used if you want to sort data into a particular directory structure after checkout"
	print "\tEXAMPLE"
	print "--sort-output=%e,preop,%a will place each output file into <output-directory>/<episodeid of each data file>/preop/<acquisitionid of each file>"
	print
	print "--match-input-file=<filename> this is the file that contains the list of the previously checked out volumes from dt_checkout_pattern.py"
	print " if you use --match-input-file=- then the matches will be read from standard input"

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:h", ["database-directory=", "output-directory=", "output-pattern=", "output-format=", "dti-output-format=", "sort-output=", "match-input-file="])

	databaseDirectory = None
	outputDirectory = None
	outputFormat = None
	outputPattern = None
	DTIOutputFormat = None
	sortOutputKeys = None
	matchInputFile = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--output-directory':
			outputDirectory = a
		if o == '--output-format':
			outputFormat = a
		if o == '--output-pattern':
			outputPattern = a
		if o == '--dti-output-format':
			DTIOutputFormat = a
		if o == '--sort-output':
			sortOutputKeys = a
		if o == '--match-input-file':
			matchInputFile = a
		if o == '-h':
			usage()
			quit()
	#for o, a in opts:
	
	if DTIOutputFormat == None:
		DTIOutputFormat = "fsl"
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if outputDirectory == None or outputFormat == None or outputPattern == None or matchInputFile == None:
		print "Mandatory argument not given"
		usage()
		quit()
	
	projectName = projectName.strip('/')
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "project directory was not found"
		quit()

	
	if outputFormat.lower() != "nifti" and outputFormat.lower() != "minc":
		print "output format not supported: " + outputFormat
		quit()

	if not os.path.isdir(outputDirectory):
		print "Output directory specified does not exist or is not a directory: " + outputDirectory
		quit()

	matchedDataResourceArray = []
	if matchInputFile == '-': # read from standard input
		inputFileStream = sys.stdin
	else:
		if os.path.isfile(matchInputFile):
			inputFileStream = file(matchInputFile, 'r')
		else:
			print matchInputFile + " doesnt exist"
			quit()

	for curLine in inputFileStream.xreadlines():
		(projectID, subjectTypeID, subjectID, visitID, studyID, episodeID, acquisitionID, dataResourceID) = curLine.split("\t")
		dataResourceID = dataResourceID.strip()
		matchedDataResourceArray.append(DataTransferPatterns.MatchedDataResource(projectID=projectID, subjectType=subjectTypeID, subjectID=subjectID, visitID=visitID, studyID=studyID, episodeID=episodeID, acquisitionID=acquisitionID, dataResourceID=dataResourceID))
	inputFileStream.close()

	idx = 0

	for curMatchedDataResource in matchedDataResourceArray:
		#print curMatchedDataResource
		idx = idx + 1

		curOutputFileName = curMatchedDataResource.metaDataPatternString(outputPattern, idx)
		if sortOutputKeys != None:
			sortOutputStrings = []
			for curOutputKey in sortOutputKeys:
				curMatchedDataResource.metaDataPatternString(pattern=curOutputKeyString, idx=idx)
				sortOutputStrings.append(curOutputKeyString)
			curOutputFileName = os.path.join(outputDirectory, os.sep.join(sortOutputStrings), curOutputFileName)
			if dryRun == False:
				try:
					os.makedirs(os.path.join(outputDirectory, os.sep.join(sortOutputStrings)))
				except os.error, e:
					if e.errno != errno.EEXIST:
						raise
		else:
			curOutputFileName = os.path.join(outputDirectory, curOutputFileName)
		#print "checking out: " + projectName + " " + curDataResourceNode.get('subjectGroupID') + " " + curDataResourceNode.get('subjectID').zfill(DataTransferConstants.padZerosLength) + " " + curDataResourceNode.get('visitID') + " " + curDataResourceNode.get('episodeID') + " " + curDataResourceNode.get('acquisitionID').zfill(DataTransferConstants.padZerosLength) + " to " + curOutputFileName
		#curDataResourceIdx = curDataResourceIdx + 1
		commandString = "dt_checkout_volume.py -p " + curMatchedDataResource.projectID + \
		" --subject-id=" + curMatchedDataResource.subjectID + \
		" --visit-id=" + curMatchedDataResource.visitID + \
		" --study-id=" +  curMatchedDataResource.studyID + \
		" --episode-id=" + curMatchedDataResource.episodeID + \
		" --acquisition-id=" + curMatchedDataResource.acquisitionID + \
		" --data-resource-id=" + DataTransferConstants.escapeDataResourceIDForBash(curMatchedDataResource.dataResourceID) + \
		" -o " + curOutputFileName + \
		" --output-format=" + outputFormat + \
		" --dti-output-format=" + DTIOutputFormat
		if dryRun == False:
			os.system(commandString)
	#	commandString = "dt_checkout_volume.py -p " + projectName + " --subject-id=" + curDataResourceNode.get('subjectID') + " --visit-id=" + curDataResourceNode.get('visitID') + " --study-id=" +  curDataResourceNode.get('studyID') + " --episode-id=" + curDataResourceNode.get('episodeID') + " --acquisition-id=" + curDataResourceNode.get('acquisitionID') + " --data-resource-id=" + DataTransferConstants.escapeDataResourceIDForBash(curDataResourceNode.get('ID')) + " -o " + curOutputFileName + " --output-format=" + outputFormat + " --dti-output-format=" + DTIOutputFormat
		#os.system(commandString)
	#for curMatchedDataResource in MatchedDataResourceArray
#def main():

if __name__ == "__main__":
	main()

