#!/usr/bin/python

# this program is a wrapper script that allows users to check out
# multiple datasets from a project using a pattern
# for example, one may wish to select the same episode across all subjects in a project to obtain all structural images

from lxml import etree
import sys
import os
import errno
import getopt
import DataTransferConstants
import DataTransferPatterns
import re
import shutil

def usage():
	print "Usage: " + sys.argv[0] + " -p <project name> [--database-directory=<>] --output-directory=<> --output-format=<> --output-pattern=<> --dti-output-format=<fsl or mrtrix> --sort-output=sort1,sort2... [--subject-type-pattern=<> --subject-pattern=<> --visit-pattern=<> --study-pattern=<> --episode-pattern=<> --acquisition-pattern=<> --data-resource-pattern=<>] [--match-output-file=] [--dry-run] [-h]"

	print
	print "Extracts and converts subsets of project data based on user-specified patterns that match metadata elements"
	print
	print "Mandatory arguments:"
	print "-p: the project name, this directory must exist: " + os.path.join("<database directory>", "<project name>")

	print "Optional arguments:"
	print "--database-directory: the root directory of the database. The following rules are used, in sequence, to determine the root directory of the database"
	print "\tThe --database-directory option."
	print "\tThe environment variable DT_PROJECT_DIR"
	print "\tThe site-specific default database directory, which is: " + DataTransferConstants.databaseDirectory
	print "--dry-run: If specified then no files actually be checked out"
	print "-v: the extraction commands will be printed as data are checked out"
	print "-h: prints this help message, program exits"

	print "Options that select datasets for extraction"
	print "--subject-type-pattern=<> --subject-pattern=<> --visit-pattern=<> --study-pattern=<> --episode-pattern=<> --acquisition-pattern=<> --data-resource-pattern=<> --user-fields-pattern=<>"

	print "All pattern options bar the final option (--user-fields-pattern) allow matching of the values of that ID, e.g. --subject-pattern refers to the numeric subject ID metadata fields in the XCEDE specification"
	print "The format of each pattern is dependent on the format of the ID. The ID formats are either string (characters) or numeric and their formats are as follows:"
	
	print "String fields (study-pattern, episode-pattern) permits the use of wildcard patterns in the same way as used in the shell for file matching"
	print ""
	print "Numeric fields (subject-pattern, visit-pattern, acquisition-pattern) uses MATLAB-like syntax"
	print "\t--subject-pattern=lowerLimit:increment:upperLimit, lowerLimit:*, *:upperlimit"
	print "\t\t so --subject-pattern=1:5:21 will select subjects with IDs 1, 6, 11, 16, 21"
	print "\t\t so --subject-pattern=*:21 will select subjects with IDs below and including 21"
	print "\t\t so --subject-pattern=21:* will select subjects with IDs with 21 or above"
	print "Use commas to specify a series of patterns: 1,4:5,7,9,2 ..."
	
	print
	print "The option --user-fields-pattern matches metadata that are associated with each ID. The required format of this argument is:"
	print "\t--user-fields-pattern=level,name,pattern,level,name,pattern..."
	print "\t\twhere level is the XCEDE node that the metadata is associated with, name is the name of the metadata field and pattern is the string used to match values of the metadata"
	print "An example is the hash field, which is used to uniquely identify subjects. The hash field's level is subject and its name is hash and the pattern would need to be appropriate for a string field, given the string format of the hash field. In order to match this field, you would need --user-fields-pattern=subject,hash,****"
	

	print
	print "Options that determine where data is extracted to and how it is formatted"
	print
	print "The data will be extracted as files and the following options determine the format and the naming scheme for these files"
	
	print 
	print "--output-directory: the directory to place the output files"
	print "--output-format: the format of the extracted files. The following formats are supported: " + ", ".join(DataTransferConstants.supportedOutputFormats()
	
print "--output-pattern: each file will be named according to this pattern. Special sequences of characters can be used to substitute IDs into the filenames as follows:"
	print ""
	print "%p: project name"
	print "%t: subject type"
	print "%s: subject ID"
	print "%v: visit ID (date)"
	print "%e: episode ID"
	print "%a: acquisition ID"
	print "%d: today's date (YYYYMMDD)"
	print "%n: the index of the file in the current set of checked out files, so the first file will be 1, second this will be 2, and so on"
	print "%%: the character %"
	print "%#: the character #"
	print
	print "User-supplied fields"
	print "#level:field# will be replaced by the meta-data from the field \"field\" at the level \"level\"."
	print "level may be subjectInfo, visit, study, episode, acquisition"
	print "Fields that are not part of XCEDE must have been inserted using dt_userfields_gui.py"
	print
	print "--sort-output=key1,key2,key3... is used to place output files in subdirectories of the output directory (--output-directory)"
	print " refer to the substitution rules for --output-pattern for how to insert the meta-data into the directory names"
	print " the extra key %, is used here to insert a comma into the directory name"
	print " This feature is intended to be used if you want to sort data into a particular directory structure after checkout"
	print "\tEXAMPLE"
	print "--sort-output=%e,preop,%a will place each output file into <output-directory>/<episodeid of each data file>/preop/<acquisitionid of each file>"
	print
	print "IMPORTANT:"
	print "\tThe wildcard character, *, is interpreted by the shell prior to program execution and thus may return a list of files in the current directory"
	print "\tTo avoid this, escape any wildcards with two backslashes, \\\\*, so that bash does not perform file matching"
	print
	print "Diffusion weighted images - B-Value and B-Vector data (--dti-output-format)"
	print "If a diffusion-weighted image is checked out then so to will its B-value and B-vector data. The format of this data may be controlled with the --dti-output-format= option, in order to produce data that is compatible with analysis programs"i
	print "Specifically, assume that dwi.nii was the diffusion weighted file checked out, then:"
	print "\t--dti-output-format=fsl (default) will produce a foo.bval and foo.bvecs file suitable for processing with FSL"
	print "\t--dti-output-format=mrtrix will produce foo.grad, which contains the B-Values and B-vectors, formatted to be compatible with the mrtrix software package"
	print
	print "Saving lists of checked out files"
	print
	print "--match-output-file=<filename> prints out the checked out volumes to a file so you can check out the same list of files again or use this to"
	print " make lists of files across multiple projects and check them out all at once"
	print " if you use --match-output-file=- then the matches will be printed to standard output"
	print "each match is printed to a line in the output file with the following format:"
	print "project id\tsubject type id\tsubject id\tvisit id\tstudy id\tepisode id\tacquisition id\tdata resource id"

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:hv", ["database-directory=", "subject-type-pattern=", "subject-pattern=", "visit-pattern=", "study-pattern=", "episode-pattern=", "acquisition-pattern=", "data-resource-pattern=", "user-fields-pattern=", "output-directory=", "output-pattern=", "output-format=", "dti-output-format=", "sort-output=", "match-output-file=", "dry-run"])

	projectName = None
	databaseDirectory = None
	subjectTypePattern = None
	subjectPattern = None
	visitPattern = None
	studyPattern = None
	episodePattern = None
	acquisitionPattern = None
	dataResourcePattern = None
	userFieldsPattern = None
	outputDirectory = None
	outputFormat = None
	outputPattern = None
	DTIOutputFormat = None
	sortOutputKeys = None
	matchOutputFile = None
	dryRun = False
	verbose = False

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--subject-type-pattern':
			subjectTypePattern = a
		if o == '--subject-pattern':
			subjectPattern = a
		if o == '--visit-pattern':
			visitPattern = a
		if o == '--study-pattern':
			studyPattern = a
		if o == '--episode-pattern':
			episodePattern = a
		if o == '--acquisition-pattern':
			acquisitionPattern = a
		if o == '--data-resource-pattern':
			dataResourcePattern = a
		if o == '--user-fields-pattern':
			userFieldsPattern = a
		if o == '--output-directory':
			outputDirectory = a
		if o == '--output-format':
			outputFormat = a
		if o == '--output-pattern':
			outputPattern = a
		if o == '--dti-output-format':
			DTIOutputFormat = a
		if o == '--sort-output':
			sortOutputKeys = a
		if o == '--match-output-file':
			matchOutputFile = a
		if o == '--dry-run':
			dryRun = True
		if o == '-h':
			usage()
			quit()
		if o == '-v':
			verbose = True
	#for o, a in opts:
		
	if DTIOutputFormat == None:
		DTIOutputFormat = "fsl"
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None or outputDirectory == None or outputFormat == None or outputPattern == None:
		print "Mandatory argument not given"
		usage()
		quit()
	projectName = projectName.strip('/')
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "project directory could not be found"
		quit()
	
	if DataTransferConstants.isOutputFormatSupported(outputFormat) == False:
		print "Output format \"" + outputFormat + "\" not supported"
		quit()

	if not os.path.isdir(outputDirectory):
		print "Output directory specified does not exist or is not a directory: " + outputDirectory
		quit()

	# verify that all the output keys are correct
	# i.e. there are no repeats and they are all valid keys
	#if sortOutputKeys != None:
	#	sortOutputKeys = sortOutputKeys.split(',')
	#	sortOutputKeysValid = True
		# make sure that all keys are valid
	#	for idx in range(len(sortOutputKeys)):
	#		if not (sortOutputKeys[idx].lower() == 'subjecttype' or sortOutputKeys[idx].lower() == 'subjectid' or sortOutputKeys[idx].lower() == 'visitid' or sortOutputKeys[idx].lower() == 'studyid' or sortOutputKeys[idx].lower() == 'episodeid' or sortOutputKeys[idx].lower() == 'acquisitionid'):
	#			sortOutputKeysValid = False
		# make sure there are no duplicate keys
	#	for idx1 in range(len(sortOutputKeys) - 1):
	#		for idx2 in range(idx1 + 1, len(sortOutputKeys) - 2):
	#			if sortOutputKeys[idx1] == sortOutputKeys[idx2]:
	#				sortOutputKeysValid = False
	#	if sortOutputKeysValid == False:
	#		print "At least one of the output keys are invalid or there is a repeated key"
	matchedDataResourceArray = DataTransferPatterns.matchPatternsInXML(projectName = projectName, databaseDirectory = databaseDirectory, subjectPattern = subjectPattern, subjectTypePattern = subjectTypePattern, visitPattern = visitPattern, studyPattern = studyPattern, episodePattern = episodePattern, acquisitionPattern = acquisitionPattern, dataResourcePattern = dataResourcePattern, userFieldsPattern = userFieldsPattern)
	#print matches
	idx = 1;
	for curMatchedDataResource in matchedDataResourceArray:
		curOutputFileName = curMatchedDataResource.metaDataPatternString(outputPattern)
		if curOutputFileName == None:
			print "There was a missing metadata field, so the following dataResource was not checked out"
			print curMatchedDataResource
		else:
			if sortOutputKeys != None:
				curOutputDirectory = curMatchedDataResource.metaDataDirectoryString(pattern = sortOutputKeys)
				curOutputFileName = os.path.join(outputDirectory, curOutputDirectory, curOutputFileName)
			else:
				curOutputFileName = os.path.join(outputDirectory, curOutputFileName)
			if dryRun == False:
				(head, tail) = os.path.split(curOutputFileName)
				try:
					os.makedirs(head)
				except os.error, e:
					if e.errno != errno.EEXIST:
						raise
			#print "checking out: " + projectName + " " + curDataResourceNode.get('subjectGroupID') + " " + curDataResourceNode.get('subjectID').zfill(DataTransferConstants.padZerosLength) + " " + curDataResourceNode.get('visitID') + " " + curDataResourceNode.get('episodeID') + " " + curDataResourceNode.get('acquisitionID').zfill(DataTransferConstants.padZerosLength) + " to " + curOutputFileName
			#curDataResourceIdx = curDataResourceIdx + 1
			
			#commandString = "/home/addo/trunk/dt_checkout_volume.py -p " + curMatchedDataResource.getProjectID() + \
			commandString = "dt_checkout_volume.py -p " + curMatchedDataResource.getProjectID() + \
			" --subject-id=" + str(curMatchedDataResource.getSubjectID()) + \
			" --visit-id=" + curMatchedDataResource.getVisitID() + \
			" --study-id=" +  curMatchedDataResource.getStudyID() + \
			" --episode-id=" + DataTransferConstants.escapeDataResourceIDForBash(curMatchedDataResource.getEpisodeID()) + \
			" --acquisition-id=" + str(curMatchedDataResource.getAcquisitionID()) + \
			" --data-resource-id=" + DataTransferConstants.escapeDataResourceIDForBash(curMatchedDataResource.getDataResourceID()) + \
			" -o " + DataTransferConstants.escapeDataResourceIDForBash(curOutputFileName) + \
			" --output-format=" + outputFormat + \
			" --dti-output-format=" + DTIOutputFormat + \
			" --database-directory=" + databaseDirectory
			#commandString = "dt_checkout_volume.py -p " + projectName + " --subject-id=" + curDataResourceNode.get('subjectID') + " --visit-id=" + curDataResourceNode.get('visitID') + " --study-id=" +  curDataResourceNode.get('studyID') + " --episode-id=" + curDataResourceNode.get('episodeID') + " --acquisition-id=" + curDataResourceNode.get('acquisitionID') + " --data-resource-id=" + DataTransferConstants.escapeDataResourceIDForBash(curDataResourceNode.get('ID')) + " -o " + curOutputFileName + " --output-format=" + outputFormat + " --dti-output-format=" + DTIOutputFormat
			if verbose == True:
				print commandString
			if dryRun == False:
				os.system(commandString)
		#if curOutputFileName == None:
	#for curDataResourceNode in dataResourceNodes:
	if matchOutputFile != None:
		if matchOutputFile == "-": # print to stdout
			for curMatchedDataResource in matchedDataResourceArray:
				print curMatchedDataResource.patternMatchString()
		else:
			fp = file(matchOutputFile, 'w')
			for curMatchedDataResource in matchedDataResourceArray:
				fp.write(curMatchedDataResource.patternMatchString() + "\n")
			
#def main():

if __name__ == "__main__":
	main()
