#!/usr/bin/python

# extracts a volume from a raw data resource in the database
# the raw data resource could be DICOM, GENESIS or ANALYZE

import tempfile

import platform

from lxml import etree
import sys
import os
import getopt
import DataTransferConstants
import re
import shutil
import errno

import dicom

from numpy import *
import struct


def usage():
	print "Usage: " + sys.argv[0] + " -p <project id> --subject-id=<> --visit-id=<> --study-id=<> --episode-id=<> --acquisition-id=<> --data-resource-id=<> -o <output file> --output-format=<> --dti-output-format=<>"
	print "Relevant values must be placed into the positions indicated by <>"
	print
	print "--dti-output-format = either fsl (default) or mrtrix"
	print " if this is fsl, then <output file>.bval and <output file>.bvecs will be generated"
	print " if this is mrtrix, then <output file>.grad will be generated"

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:o:h", ["database-directory=", "subject-id=", "visit-id=", "study-id=", "episode-id=", "acquisition-id=", "data-resource-id=", "output-format=", "dti-output-format="])
	
	projectName = None
	databaseDirectory = None
	subjectID = None
	visitID = None
	studyID = None
	episodeID = None
	acquisitionID = None
	dataResourceID = None
	outputFormat = None
	outputFileName = None
	DTIOutputFormat = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--subject-id':
			subjectID = a
		if o == '--visit-id':
			visitID = a
		if o == '--study-id':
			studyID = a
		if o == '--episode-id':
			episodeID = a
		if o == '--acquisition-id':
			acquisitionID = a
		if o == '--data-resource-id':
			dataResourceID = a
		if o == '--output-format':
			outputFormat = a
		if o == '--dti-output-format':
			DTIOutputFormat = a
		if o == '-o':
			outputFileName = a
		if o == "-h":
			usage()
			quit()
	#for o, a in opts:
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	#print projectName 
	#print databaseDirectory 
	#print subjectID 
	#print visitID 
	#print studyID 
	#print episodeID 
	#print acquisitionID 
	#print dataResourceID 
	#print outputFormat 
	#print outputFileName 
	#print DTIOutputFormat 

	head, tail) = os.path.split(sys.argv[0])
	scriptPath = head 
	
	machineType = platform.machine()

	if machineType == 'x86_64':
		DCMTONII = os.path.join(scriptPath, 'dcm2nii.x86_64')
	elif machineType == 'i686':
		DCMTONII = os.path.join(scriptPath, 'dcm2nii.x86')
	else:
		DCMTONII = DataTransferConstants.findCommand("dcm2nii")
		if DCMTONII == None:
			print "dcm2nii was not found"
			quit()
	
	#DICOMTOMINC = DataTransferConstants.findCommand("dicom3_to_minc")
	#if DICOMTOMINC == None:
	#	print "dicom3_to_minc was not found"
	#	quit()
	
	if DataTransferConstants.isOutputFormatSupported(outputFormat) == False:
		print "Output format \"" + outputFormat + "\" not supported"
		quit()

	if projectName == None or subjectID == None or visitID == None or studyID == None or episodeID == None or acquisitionID == None or dataResourceID == None or outputFileName == None:
		print "Mandatory argument not given"
		usage()
		quit()

	projectName = projectName.strip('/')
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "project directory was not found"
		quit()


	if DTIOutputFormat == None:
		DTIOutputFormat = "fsl"

	# read in project XML file
	if not os.path.isfile(os.path.join(databaseDirectory, projectName, "projects.xml")):
		print "The project's XML file could not be found: " + os.path.join(DataTransferConstants.databaseDirectory, projectName, "projects.xml")
		quit()
	else:
		projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
		projectXMLRoot = projectXML.getroot()

		# make sure that all the ids are in the project
		#print "/XCEDE/dataResource[@ID='" + dataResourceID + "' and @acquisitionID='" + acquisitionID + "' and @episodeID='" + episodeID + "' and @studyID='" + studyID + "' and @visitID='" + visitID + "' and @subjectID='" + subjectID + "' and @projectID='" + projectName + "']"
		dataResourceNodes = projectXML.xpath("/XCEDE/dataResource[@ID='" + dataResourceID + "' and @acquisitionID='" + acquisitionID + "' and @episodeID='" + episodeID + "' and @studyID='" + studyID + "' and @visitID='" + visitID + "' and @subjectID='" + subjectID + "' and @projectID='" + projectName + "']")
		if len(dataResourceNodes) != 1:
			print "There were zero or multiple matches for the dataset"
			quit()
		else:
			dataResourceNode = dataResourceNodes[0]

			# compute the directory where the file should be
			directory = DataTransferConstants.acquisitionFileDirectory(databaseDirectory, projectName, subjectID, visitID, studyID, episodeID, acquisitionID)
			if not os.path.isdir(directory):
				print "The directory that I was looking for was not accessible: " + directory
				quit()
			else:
				# the files that I want to convert are dicom or genesis files
				# these are stored in tarballs dicomfiles.tar.gz and genesisfiles.tar.gz respectively
				# create a temporary directory, extract the dicom/genesis files
				# run dcm2nii
				# copy the resultant nifti file to the id given on the command line
				# delete the temporary directory
				if dataResourceNode.get('format') == "GENESIS" and os.path.isfile(os.path.join(directory, "genesisfiles.tar.gz")):
					pass
				elif dataResourceNode.get('format') == "DICOM" and os.path.isfile(os.path.join(directory, "dicomfiles.tar.gz")):
					# make sure that the files in the tarball don't have directories associated with them
					
					curSeriesFiles = []
					for curURINode in dataResourceNode.iterchildren(tag = "uri"):
						curSeriesFiles.append(curURINode.text)
					
					TarListFile = tempfile.NamedTemporaryFile(mode = 'w+t')
					TarListFile.write("\n".join(["%s" % (v) for v in curSeriesFiles]))
					TarListFile.flush()
					TarListFileName = TarListFile.name
					tempDirectory = tempfile.mkdtemp()
					# now append the temporary directory to the files
					
					for curIdx in range(len(curSeriesFiles)):
						curSeriesFiles[curIdx] =  os.path.join(tempDirectory, curSeriesFiles[curIdx])
					commandString = 'tar xzf "' + os.path.join(directory, "dicomfiles.tar.gz") + "\" -C " + tempDirectory + " -T " + TarListFileName
					os.system(commandString)
					TarListFile.close()

					# strip the extension from the output file name
					pat = re.compile("^(.*)\.(nii|mnc|mgh|dcm)$")
					match = pat.match(outputFileName)
					if match != None:
						outputFileName = match.group(1)
					if outputFormat.lower() == 'dicom':
						# easy, no conversion required
						try:
							os.makedirs(outputFileName)
						except os.error, e:
							if e.errno == errno.EEXIST:
								pass
							if e.errno != errno.EEXIST:
								raise

						for curFile in curSeriesFiles:
							(head, tail) = os.path.split(curFile)
							shutil.copyfile(curFile, os.path.join(outputFileName, tail))
							
					elif outputFormat.lower() == 'nifti':
						#commandString = DCMTONII + " -g n " + " ".join(curSeriesFiles)
						commandString = DCMTONII + " -g n " + curSeriesFiles[0]
						#print commandString
						DCMPipe = os.popen(commandString, 'r')
						DCMFiles = DCMPipe.readlines()
						DCMPipe.close()
						
						for curLine in range(len(DCMFiles)):
							pat = re.compile("^.+?->(.+)\.nii$")
							DCMFiles[curLine] = DCMFiles[curLine].rstrip()
							#print DCMFiles[curLine]
							match = pat.match(DCMFiles[curLine])
							if match != None:
								DCMToNIIOutputFile = match.group(1)
								#print "output file: " + DCMToNIIOutputFile
								break
						shutil.copyfile(os.path.join(tempDirectory, DCMToNIIOutputFile + ".nii"), outputFileName + ".nii")
					#elif outputFormat.lower() == 'minc':
					#	otherTempDirectory = tempfile.mkdtemp()
					#	commandString = DICOMTOMINC + " " + otherTempDirectory + " " + " ".join(curSeriesFiles) + " 2>&1 > /dev/null"
					#	os.system(commandString)
						# look for mnc files in the otherTempDirectory
					#	for root, dirs, files in os.walk(otherTempDirectory):
					#		for name in files:
					#			shutil.copyfile(os.path.join(otherTempDirectory, name), outputFileName + ".mnc")

					#	shutil.rmtree(otherTempDirectory)
					elif outputFormat.lower() == 'mgh': # the mgh format for freesurfer
						# TODO: write
						# get the dimension directions
						sliceOrientationX = [0, 0, 0]
						sliceOrientationY = [0, 0, 0]
						sliceOrientationZ = [0, 0, 0]
						
						numRows = 0
						numColumns = 0
						numSlices = 0
						numVolumes = 1
						
						voxelDimensions = [0, 0, 0, 0]
						mosaicFormat = False

						for curDimensionNode in dataResourceNode.iterchildren(tag = 'dimension'):
							if curDimensionNode.get('label') == 'x':
								for directionNode in curDimensionNode.iterchildren(tag = 'direction'):
									sliceOrientationX = directionNode.text.split(" ")
								for sizeNode in curDimensionNode.iterchildren(tag = 'size'):
									numColumns = int(sizeNode.text)
								for spacingNode in curDimensionNode.iterchildren(tag = 'spacing'):
									voxelDimensions[0] = float(spacingNode.text)
								for idx in range(len(sliceOrientationX)):
									sliceOrientationX[idx] = float(sliceOrientationX[idx])
							elif curDimensionNode.get('label') == 'y':
								for directionNode in curDimensionNode.iterchildren(tag = 'direction'):
									sliceOrientationY = directionNode.text.split(" ")
								for sizeNode in curDimensionNode.iterchildren(tag = 'size'):
									numRows = int(sizeNode.text)
								for spacingNode in curDimensionNode.iterchildren(tag = 'spacing'):
									voxelDimensions[1] = float(spacingNode.text)
								for idx in range(len(sliceOrientationY)):
									sliceOrientationY[idx] = float(sliceOrientationY[idx])
							elif curDimensionNode.get('label') == 'z':
								for directionNode in curDimensionNode.iterchildren(tag = 'direction'):
									sliceOrientationZ = directionNode.text.split(" ")
								# find out if we have mosaic'd data
								if curDimensionNode.get('splitRank') != None:
									mosaicFormat = True
									for sizeNode in curDimensionNode.iterchildren(tag = 'size'):
										numInMosaic = int(sizeNode.text)
									sliceIndices = curDimensionNode.get('outputSelect')
									if sliceIndices != None:
										sliceIndexArray = sliceIndices.split(" ")
										numSlices = len(sliceIndexArray)
								else:
									for sizeNode in curDimensionNode.iterchildren(tag = 'size'):
										numSlices = int(sizeNode.text)
								for spacingNode in curDimensionNode.iterchildren(tag = 'spacing'):
									voxelDimensions[2] = float(spacingNode.text)
								for idx in range(len(sliceOrientationZ)):
									sliceOrientationZ[idx] = float(sliceOrientationZ[idx])
							elif curDimensionNode.get('label') == 't':
								for sizeNode in curDimensionNode.iterchildren(tag = 'size'):
									numVolumes = int(sizeNode.text)
								for spacingNode in curDimensionNode.iterchildren(tag = 'spacing'):
									voxelDimensions[3] = float(spacingNode.text)
							# get the orientation information from the DICOM headers
						try:
							ds = dicom.filereader.read_file(os.path.join(tempDirectory, curSeriesFiles[0]), stop_before_pixels=True)
						except Exception, e:
							print "one of the dicom files was corrupted: " + os.path.join(tempDirectory, curSeriesFiles[0])
							exit()

						# DICOM is LPS, whereas MGH is RAS, so flip the X and Y components of each of the slice directions
						sliceOrientationX[0] = -sliceOrientationX[0]
						sliceOrientationX[1] = -sliceOrientationX[1]
						sliceOrientationY[0] = -sliceOrientationY[0]
						sliceOrientationY[1] = -sliceOrientationY[1]
						sliceOrientationZ[0] = -sliceOrientationZ[0]
						sliceOrientationZ[1] = -sliceOrientationZ[1]
	
						if mosaicFormat == True:
							mosaicSliceOrientationX = {}
							mosaicSliceOrientationY = {}
							mosaicSliceOrientationZ = {}

							for curLine in ds[0x0029, 0x1020].value.splitlines():
								#Sag (x), Cor (y), and Tra (z)
								match = re.match("^sSliceArray.asSlice\[(\d+)\].sPosition.dSag\s+=\s+([+-]?\d+(\.\d+)?)$", curLine);
								if match != None:
									mosaicSliceOrientationX[int(match.group(1))] = -float(match.group(2))
								match = re.match("^sSliceArray.asSlice\[(\d+)\].sPosition.dCor\s+=\s+([+-]?\d+(\.\d+)?)$", curLine);
								if match != None:
									mosaicSliceOrientationY[int(match.group(1))] = -float(match.group(2))
								match = re.match("^sSliceArray.asSlice\[(\d+)\].sPosition.dTra\s+=\s+([+-]?\d+(\.\d+)?)$", curLine);
								if match != None:
									mosaicSliceOrientationZ[int(match.group(1))] = float(match.group(2))
							
							if numSlices % 2 == 0:
								volMean = [mosaicSliceOrientationX[numSlices / 2], mosaicSliceOrientationY[numSlices / 2], mosaicSliceOrientationZ[numSlices / 2]] 
							else:
								volMean = [ \
								(mosaicSliceOrientationX[numSlices / 2] + mosaicSliceOrientationX[numSlices / 2 + 1]) / 2, \
								(mosaicSliceOrientationY[numSlices / 2] + mosaicSliceOrientationY[numSlices / 2 + 1]) / 2, \
								(mosaicSliceOrientationZ[numSlices / 2] + mosaicSliceOrientationZ[numSlices / 2 + 1]) / 2]
							imageCentre = [float(numColumns) / 2, float(numRows) / 2, float(numSlices) / 2]
							sliceOrientationArray = array([ \
							[sliceOrientationX[0], sliceOrientationY[0], sliceOrientationZ[0], volMean[0]], \
							[sliceOrientationX[1], sliceOrientationY[1], sliceOrientationZ[1], volMean[1]], \
							[sliceOrientationX[2], sliceOrientationY[2], sliceOrientationZ[2], volMean[2]], \
							[                   0,                    0,                    0,          1]])
							
							origin = mat(sliceOrientationArray) * mat(array([-voxelDimensions[0] * imageCentre[0], -voxelDimensions[1] * imageCentre[1], -voxelDimensions[2] * imageCentre[2], 1]).reshape(4, 1))
							del sliceOrientationArray	
						else:
							origin = ds[0x0020,0x0032].value
							# DICOM is LPS, whereas MGH is RAS, so flip the X and Y components of each of the slice directions
							origin[0] = -origin[0]
							origin[1] = -origin[1]

							origin = array(origin)
							#print sliceOrientationZ
							#sliceOrientationZ[0] = sliceOrientationX[1] * sliceOrientationY[2] - sliceOrientationX[2] * sliceOrientationY[1]
							#sliceOrientationZ[1] = sliceOrientationX[2] * sliceOrientationY[0] - sliceOrientationX[0] * sliceOrientationY[2]
							#sliceOrientationZ[2] = sliceOrientationX[0] * sliceOrientationY[1] - sliceOrientationX[1] * sliceOrientationY[0]
							#print sliceOrientationZ
						
						del ds
						print "volume size: " + str(numRows) + ", " + str(numColumns) + ", " + str(numSlices) + ", " + str(numVolumes)
						VoxToRAS = array([ \
						[sliceOrientationX[0], sliceOrientationY[0], sliceOrientationZ[0]], \
						[sliceOrientationX[1], sliceOrientationY[1], sliceOrientationZ[1]], \
						[sliceOrientationX[2], sliceOrientationY[2], sliceOrientationZ[2]]])
						#print "volume spacing: " + str(voxelDimensions)
						#VoxToRAS = array([ \
						#[sliceOrientationX[0] * voxelDimensions[0], sliceOrientationY[0] * voxelDimensions[1], sliceOrientationZ[0] * voxelDimensions[2], 0], \
						#[sliceOrientationX[1] * voxelDimensions[0], sliceOrientationY[1] * voxelDimensions[1], sliceOrientationZ[1] * voxelDimensions[2], 0], \
						#[sliceOrientationX[2] * voxelDimensions[0], sliceOrientationY[2] * voxelDimensions[1], sliceOrientationZ[2] * voxelDimensions[2], 0], \
						#[0                   , 0                   , 0                   , 1]])
						#print origin
						#print voxelDimensions
						#print VoxToRAS
						
						# write the file out
						fp = open(outputFileName + ".mgh", 'wb')
						# magic number
						fp.write(struct.pack('>i', 1))
						# now its the volume dimensions
						fp.write(struct.pack('>i', numColumns))
						fp.write(struct.pack('>i', numRows))
						fp.write(struct.pack('>i', numSlices))
						fp.write(struct.pack('>i', numVolumes))
						# this is the code for a float field, we do not support tensor images
						fp.write(struct.pack('>i', 3))
						# this is a DOF field, we just use 1
						fp.write(struct.pack('>i', 1))
						
						unusedSpaceSize = 256
						usedSpaceSize = 60 # (3 * 4 + 4 * 3 * 4) = (12 + 48)

						# ras_good_flag = 1
						fp.write(struct.pack('>h', 1))
						# now write the voxel dimensions
						for curDim in range(3):
							fp.write(struct.pack('>f', voxelDimensions[curDim]))
						# now write VoxToRAS
						for curElement in (VoxToRAS.transpose()).flat:
							fp.write(struct.pack('>f', curElement))
						# now write origin
						for curElement in origin.flat:
							fp.write(struct.pack('>f', curElement))
						# dummy to write 254 - 60
						for curElement in range(254 - 60):
							fp.write(struct.pack('>c', '\x00'))

						for curFile in curSeriesFiles:
							try:
								curDS = dicom.filereader.read_file(os.path.join(tempDirectory, curFile), stop_before_pixels=False)
							except Exception, e:
								print "one of the dicom files was corrupted: " + os.path.join(tempDirectory, curSeriesFiles[0])
								exit()
							if mosaicFormat == True:
								
								for curSlice in range(numSlices):
									curRow = curSlice / numInMosaic
									curColumn = curSlice % numInMosaic

									curSlicePixelArray = curDS.PixelArray[(curRow * numRows):((curRow + 1) * numRows), (curColumn * numColumns):((curColumn + 1) * numColumns)]
									if sys.byteorder == 'little':
										curSlicePixelArray.byteswap(True)
									s = curSlicePixelArray.tostring()
									fp.write(s)
									del s

									#for curElement in curSlicePixelArray.flat:
									#	arrayToWrite.append(struct.pack('>f', float(curElement)))
							else:
								if sys.byteorder == 'little': # i'm assuming that pydicom has returned the data in machine endian format
									curDS.PixelArray.byteswap(True)
								s = curDS.PixelArray.tostring()
								fp.write(s)
								del s

								#for curElement in curDS.PixelArray.flat:
								#	arrayToWrite.append(struct.pack('>f', curElement))
						#arrayToWrite.tofile(fp)
						fp.write(struct.pack('>f', 1))
						fp.write(struct.pack('>f', 1))
						fp.write(struct.pack('>f', 1))
						fp.write(struct.pack('>f', 1))
						
						fp.close()
						#if outputFormat.lower() == 'nifti':
					shutil.rmtree(tempDirectory)

					# look for B-Values and B-Vectors in the file and export these
					acquisitionNodes = projectXML.xpath("/XCEDE/acquisition[@ID='" + acquisitionID + "' and @episodeID='" + episodeID + "' and @studyID='" + studyID + "' and @visitID='" + visitID + "' and @subjectID='" + subjectID + "' and @projectID='" + projectName + "']")
					if len(acquisitionNodes) != 1:
						print "WARNING: there was no acquisition node associated with the dataResource node"
					else:
						BValueNode = None
						BVectorsNode = None
						for curNode in acquisitionNodes[0].iterchildren():
							if curNode.tag == "BValues":
								BValueNode = curNode
							elif curNode.tag == "BVectors":
								BVectorsNode = curNode

						if BValueNode != None and BVectorsNode != None:
							BValues = BValueNode.text.split(',')
							#for curBValueIdx in range(len(BValues)):
							#	BValues[curBValueIdx] = float(BValues[curBValueIdx])
							BVectorsFirstSplit = BVectorsNode.text.split(';')
							BVectors = []
							for idx in range(len(BVectorsFirstSplit)):
								BVectors.append(BVectorsFirstSplit[idx].split(','))

							for idx in range(len(BVectors)):
								BVectors[idx][0] = float(BVectors[idx][0])
								BVectors[idx][1] = float(BVectors[idx][1])
								BVectors[idx][2] = float(BVectors[idx][2])

							if DTIOutputFormat.lower() == "fsl": 
#<dimension label="x">
#	<size>240</size>
#	<spacing>0.6875</spacing>
#	<gap>0</gap>
#	<units>mm</units>
#	<direction>0.999847359428 0.0002397189313 -0.0174699850816</direction>
								
								# make a bvecs and a bval file
								# bval file is one row of space separated values
								BValuesFile = file(outputFileName + ".bvals", "w")
								BValuesFile.write(" ".join(BValues))
								BValuesFile.close()
								BVectorsFile = file(outputFileName + ".bvecs", "w")
								sliceOrientationX = [1, 0, 0]
								sliceOrientationY = [0, 1, 0]
								sliceOrientationZ = [0, 0, 1]

								# we to do a dot product
								for curDimensionNode in dataResourceNode.iterchildren(tag = "dimension"):
									# this is for FSL, may transfer this to dt_checkout_volume
									if curDimensionNode.get("label") == 'x':
										for curOrientationNode in curDimensionNode.iterchildren(tag = 'direction'):
											sliceOrientationX = curOrientationNode.text.split(" ")
											sliceOrientationX[0] = float(sliceOrientationX[0])
											sliceOrientationX[1] = float(sliceOrientationX[1])
											sliceOrientationX[2] = float(sliceOrientationX[2])
									if curDimensionNode.get("label") == 'y':
										for curOrientationNode in curDimensionNode.iterchildren(tag = 'direction'):
											sliceOrientationY = curOrientationNode.text.split(" ")
											sliceOrientationY[0] = float(sliceOrientationY[0])
											sliceOrientationY[1] = float(sliceOrientationY[1])
											sliceOrientationY[2] = float(sliceOrientationY[2])
									if curDimensionNode.get("label") == 'z':
										for curOrientationNode in curDimensionNode.iterchildren(tag = 'direction'):
											sliceOrientationZ = curOrientationNode.text.split(" ")
											sliceOrientationZ[0] = float(sliceOrientationZ[0])
											sliceOrientationZ[1] = float(sliceOrientationZ[1])
											sliceOrientationZ[2] = float(sliceOrientationZ[2])
											#correctedBVector[0] = curBVector[0] * sliceOrientationX[0] + curBVector[1] * sliceOrientationX[1] + curBVector[2] * sliceOrientationX[2]
											#correctedBVector[1] = curBVector[0] * sliceOrientationY[0] + curBVector[1] * sliceOrientationY[1] + curBVector[2] * sliceOrientationY[2]
											#correctedBVector[1] = -correctedBVector[1] # to be compatible with dcm2nii, haven't figure this one out yet
											#correctedBVector[2] = curBVector[0] * sliceOrientationZ[0] + curBVector[1] * sliceOrientationZ[1] + curBVector[2] * sliceOrientationZ[2]
								 
								# this is a little trickier, the bvecs file is three lines, all X, all Y then all Z vector components
								for idx in range(len(BVectors)):
									CorrectedBVec = BVectors[idx][0] * sliceOrientationX[0] + BVectors[idx][1] * sliceOrientationX[1] + BVectors[idx][2] * sliceOrientationX[2]
									BVectorsFile.write(str(CorrectedBVec))
									if idx < (len(BVectors) - 1):
										BVectorsFile.write(" ")
									else:
										BVectorsFile.write("\n")
								for idx in range(len(BVectors)):
									CorrectedBVec = -(BVectors[idx][0] * sliceOrientationY[0] + BVectors[idx][1] * sliceOrientationY[1] + BVectors[idx][2] * sliceOrientationY[2])
									BVectorsFile.write(str(CorrectedBVec))
									if idx < len(BVectors) - 1:
										BVectorsFile.write(" ")
									else:
										BVectorsFile.write("\n")
								for idx in range(len(BVectors)):
									CorrectedBVec = BVectors[idx][0] * sliceOrientationZ[0] + BVectors[idx][1] * sliceOrientationZ[1] + BVectors[idx][2] * sliceOrientationZ[2]
									BVectorsFile.write(str(CorrectedBVec))
									if idx < len(BVectors) - 1:
										BVectorsFile.write(" ")
									else:
										BVectorsFile.write("\n")
								BVectorsFile.close()
							elif DTIOutputFormat.lower() == "mrtrix":
								BFile = file(outputFileName + ".grad", "w")
								for idx in range(len(BVectors)):
									BFile.write(str(-float(BVectors[idx][0])) + "\t" + str(-float(BVectors[idx][1])) + "\t" + str(BVectors[idx][2]) + "\t" + str(BValues[idx]) + "\n")
								BFile.close()
							#if DTIOutputFormat.lower() == "fsl":
						#if len(BValueNodes) == 1 and len(BVectorsNodes) == 1:
					#if len(acquisitionNodes) != 1:
				else:
					# copy the files verbatim
					URIChildren = dataResourceNode.findall("uri")
					print outputFileName
					if len(URIChildren) > 1:
						try:
							(head, tail) = os.path.split(outputFileName)
							os.makedirs(head)
						except os.error, e:
							if e.errno == errno.EEXIST:
								print "directory " + outputFileName + " already exists, overwriting files underneath"
							if e.errno != errno.EEXIST:
								raise
						for curChild in URIChildren:
							shutil.copyfile(os.path.join(directory, curChild.text), outputFileName)
					else:
							shutil.copyfile(os.path.join(directory, URIChildren[0].text), outputFileName)
				#if dataResourceNode.get('format') == "GENESIS" and os.path.isfile(os.path.join(directory, "genesisfiles.tar.gz")):
			#if not os.path.isdir(directory):
		#if len(dataResourceNodes) != 1:
	#if not os.path.isfile(os.path.join(DataTransferConstants.databaseDirectory, projectName, "projects.xml")):
#def main():

if __name__ == "__main__":
	main()
