#!/usr/bin/python

from lxml import etree

import re
import os
import errno
import sys
import getopt
import subprocess
import hashlib
import math
import operator
import tempfile
from struct import unpack
from StringIO import StringIO

import numpy

import DataTransferConstants
import DataTransferUserFields
import DataTransferProject

import DICOMObjects

import dicom

class subject:
	name = ''
	address = ''
	DOB = ''
	sex = ''
	species = ''
	nameEnc = ''
	addressEnc = ''
	
	visits = {}

	def __init__(self, DOB = None, sex = None, species = None, name = None, address = None, nameEnc = None, addressEnc = None):
		self.DOB = DOB
		self.sex = sex
		self.species = species
		self.name = name
		self.address = address
		self.nameEnc = nameEnc
		self.addressEnc = addressEnc

		if self.address == None:
			self.address = ''
		self.visits = {}

class visit:
	series = []
	date = ''
	def __init__(self, hash):
		self.hash = hash
	
class series:
	files = []
	seriesID = ''

#class DICOMKeys
#	def __init__(self):
#		pass

################## SPECIAL FIELDS AS REQUESTED BY AMANDA ##################################
#Report received
#Follow up required I'm putting this in at the episode level, since it may be related to particular scans
#If yes, f/up completed?
#Invoice number I'm putting this in at the study level
#Chief Investigator already in there because we have contributor info
#Consent for Future Control (yes/no) At patient level, in patientInfo

def usage():
	print "Usage: " + sys.argv[0] + " -p <project name> -d <DICOM directory> --new-subjects-type=<patients or controls> --study-id=<study> [--database-directory=<>] [--subject-species=<species>]"
	print
	print "Mandatory arguments:"
	print "-p <project name>: the project name, the project's directory must exist and be created by dt_create_project.py"
	print "-d <DICOM directory>: the root directory of the DICOM files to be imported"
	print "--new-subjects-type=<patients or controls>: the subject type for new subjects that are created from imported data"
	print "--study-id=<study id>: the study id, this will generally be something like MR_STUDY or CT_STUDY"
	print
	print "Optional arguments:"
	print "-h: help (this message)"
	print "--database-directory=<database directory>: the root directory of where the database is situated"
	print "--subject-species=<species>: the speices of the subject, defaults to \"human\""

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:d:h", ["database-directory=", "new-subjects-type=", 'subject-species=', 'study-id='])

	#DCCP = DataTransferConstants.findCommand("dccp")
	#if DCCP == None:
	#	print "Could not find the command dccp"
	#	quit()
	
	PIGZ = DataTransferConstants.findCommand("pigz")
	if PIGZ == None:
		compressionProgram = 'gzip'
	else:
		compressionProgram = 'pigz'
	# all of the numeric, incremented IDs will have their corresponding directories padded with zeros to this length

	projectName = None
	databaseDirectory = None
	baseDirectory = None
	newSubjectsType = None
	subjectSpecies = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '-h':
			usage()
			quit()
		if o == '--database-directory':
			databaseDirectory = a
		if o == '-d':
			baseDirectory = a
		if o == '--new-subjects-type':
			newSubjectsType = a
		if o == '--subject-species':
			subjectSpecies = a
		if o == '--study-id':
			curStudyID = a
	#for o, a in opts:

	temporaryDirectory = tempfile.mkdtemp()

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None or baseDirectory == None or newSubjectsType == None or curStudyID == None:
		print "Mandatory argument not given"
		usage()
		quit()
	
	if projectName != None:
		if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
			print "Project \"" + projectName + "\" was not found in the database at " + databaseDirectory 
			quit()
	
	if not (newSubjectsType == "controls" or newSubjectsType == "patients"):
		print "subject type MUST be \"patients\" or \"controls\""
		quit()

	encryptionObject = DataTransferConstants.encryptionModule(projectName = projectName, databaseDirectory = databaseDirectory, publicOrPrivate="public")

	# look for bad characters in curStudyID
	pat = re.compile('^\w+$')
	mat = pat.match(curStudyID)
	
	if mat == None:
		print "the study name contains illegal characters, it must not have any spaces in it"
		quit()

	# read in project XML file
	projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
	projectXMLRoot = projectXML.getroot()
	
	projectPatientsListRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='patients']")
	projectControlsListRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='controls']")
	
	if len(projectPatientsListRoot) == 0:
		print "No subjects subject group list, should not happen"
		quit()

	if len(projectControlsListRoot) == 0:
		print "No controls subject group list, should not happen"
		exit(1)

	nameHashMapping = DataTransferProject.NameHashMapping(databaseDirectory = databaseDirectory, projectName = projectName)
	
	subjectGroupLists = {}
	subjectGroupLists['controls'] = projectControlsListRoot[0]
	subjectGroupLists['patients'] = projectPatientsListRoot[0]

	if os.path.isdir(baseDirectory):
		subjectsToAddToNameMapping = []
		DICOMTags = DICOMUtilities.getDICOMKeysRecursive(baseDirectory, Recurse = True, keyArray = DICOMObjects.DICOMTags)
	
		subjects = {}
		newDICOMTags = {}
		# go through each file that had DICOM data and check to see what subject it was
		# what visit it was and make the subjects dictionary
		for curFile in DICOMTags.iterkeys():
			
			# check to see if critical tags are in the file, if they are not, delete the file, I'm not processing it
			if DICOMTags[curFile]['subjectName'] != None and \
				DICOMTags[curFile]['subjectDOB'] != None and \
				DICOMTags[curFile]['subjectSex'] != None and \
				DICOMTags[curFile]['acquisitionDate'] != None and \
				DICOMTags[curFile]['numRows'] != None and \
				DICOMTags[curFile]['numCols'] != None and \
				DICOMTags[curFile]['bitsAllocated'] != None and \
				DICOMTags[curFile]['seriesNumber'] != None and \
				DICOMTags[curFile]['seriesTime'] != None and \
				DICOMTags[curFile]['imageNumber'] != None and \
				DICOMTags[curFile]['seriesNumber'] != None:
				
				subjectHash = DataTransferConstants.getSubjectHash(DICOMTags[curFile]['subjectName'], DICOMTags[curFile]['subjectDOB'])

				if not subjectHash in subjects:
					print DICOMTags[curFile]['subjectName'] + " " + subjectHash
					if DICOMTags[curFile]['subjectName'] == None:
						subjectName = ""
					else:
						subjectName = DICOMTags[curFile]['subjectName']
					subjectNameEnc = encryptionObject.encryptText(subjectName)
					subjectNameEnc = subjectNameEnc.replace("\n", "")
					if DICOMTags[curFile]['subjectAddress'] == None:
						subjectAddress = ""
					else:
						subjectAddress = DICOMTags[curFile]['subjectAddress']

					subjectAddressEnc = encryptionObject.encryptText(subjectAddress)
					subjectAddressEnc = subjectAddressEnc.replace("\n", "")
					print curFile
					subjects[subjectHash] = subject(DICOMTags[curFile]['subjectDOB'], DICOMTags[curFile]['subjectSex'], "human", DICOMTags[curFile]['subjectName'], DICOMTags[curFile]['subjectAddress'], subjectNameEnc, subjectAddressEnc)

				if not DICOMTags[curFile]['acquisitionDate'] in subjects[subjectHash].visits:
					subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']] = {}

				if DICOMTags[curFile]['acquisitionDate'] in subjects[subjectHash].visits and \
					not (DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber']) in subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']]:
					subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']][(DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber'])] = {}
				
				if DICOMTags[curFile]['acquisitionDate'] in subjects[subjectHash].visits \
					and (DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber']) in subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']] and \
					not DICOMTags[curFile]['imageType'] in subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']][(DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber'])]:
					subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']][(DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber'])][DICOMTags[curFile]['imageType']] = {}
					
				if DICOMTags[curFile]['acquisitionDate'] in subjects[subjectHash].visits \
					and (DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber']) in subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']] \
					and DICOMTags[curFile]['imageType'] in subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']][(DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber'])] \
					and not DICOMTags[curFile]['imageNumber'] in subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']][(DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber'])][DICOMTags[curFile]['imageType']]:
					subjects[subjectHash].visits[DICOMTags[curFile]['acquisitionDate']][(DICOMTags[curFile]['seriesNumber'], DICOMTags[curFile]['seriesTime'], DICOMTags[curFile]['echoNumber'])][DICOMTags[curFile]['imageType']][DICOMTags[curFile]['imageNumber']] = curFile

				newDICOMTags[curFile] = DICOMTags[curFile]
				#if subjectName != None and subjectDOB != None and subjectSex != None and DICOMTags[curFile]['acquisitionDate'] != None:
			else:
				print curFile + " invalid"
			#if DICOMTags[curFile]['subjectName'] != None and \ DICOMTags[curFile]['subjectDOB'] != None and \ DICOMTags[curFile]['subjectSex'] != None and \ DICOMTags[curFile]['acquisitionDate'] != None and \ DICOMTags[curFile]['numRows'] != None and \ DICOMTags[curFile]['numCols'] != None and \ DICOMTags[curFile]['bitsAllocated'] != None and \ DICOMTags[curFile]['seriesNumber'] != None and \ DICOMTags[curFile]['imageNumber'] != None and \ DICOMTags[curFile]['seriesTime'] != None and \ DICOMTags[curFile]['seriesNumber'] != None:
		#for curFile in DICOMTags.iterkeys():
		DICOMTags = newDICOMTags
		del newDICOMTags
		filesProcessed = {}
		for curFile in DICOMTags.iterkeys():
			filesProcessed[curFile] = False

		projectURI = projectName
		for curSubject in subjects.iterkeys():
			curSubjectID = nameHashMapping.getIDByHash(hash = curSubject)
			
			if curSubjectID == None:
				# subject is not in the database
				# so add it to the name hash mapping to get a new ID
				curSubjectType = newSubjectsType

				curSubjectID = nameHashMapping.addSubject(subjectHash = curSubject, subjectName = subjects[curSubject].name, subjectSex = subjects[curSubject].sex, subjectDOB = subjects[curSubject].DOB, subjectAddress = subjects[curSubject].address)
				curSubjectRootNode = etree.SubElement(projectXMLRoot, "subject")
				curSubjectRootNode.set("ID", str(curSubjectID))
				subjects[curSubject].ID = curSubjectID
				curSubjectRootNode.set("projectID", projectName)
				curSubjectRootNode.set("subjectGroupID", newSubjectsType)
				curSubjectInfoNode = etree.SubElement(curSubjectRootNode, 'subjectInfo')
				subjectsToAddToNameMapping.append(curSubject)
				speciesNode = etree.SubElement(curSubjectInfoNode, "species")
				if subjectSpecies == None:
					speciesNode.text = "human"
				else:
					speciesNode.text = subjectSpecies

				sexNode = etree.SubElement(curSubjectInfoNode, "sex")
				sexNode.text = subjects[curSubject].sex
				DOBNode = etree.SubElement(curSubjectInfoNode, "birthdate")
				DOBNode.text = subjects[curSubject].DOB
				#ConsentNode = etree.SubElement(curSubjectInfoNode, "consentForFutureControl")
				#ConsentNode.text = "please edit"
				hashNode = etree.SubElement(curSubjectInfoNode, "hash")
				hashNode.text = curSubject

				# add the subject to the subjectGroup list
				tmpElement = etree.SubElement(subjectGroupLists[newSubjectsType], 'subjectID')
				tmpElement.text = str(curSubjectID)
			else:
				curSubjectNodes = projectXMLRoot.xpath("/XCEDE/subject[@ID='" + str(curSubjectID) + "' and @projectID='" + projectName + "']")
				if len(curSubjectNodes) != 1:
					print "there is not one subject node, this shouldnt happen"
				else:
					curSubjectNode = curSubjectNodes[0]
					curSubjectType = curSubjectNode.get("subjectGroupID")
				
			# check to see if the subject has been added to the database
			#curSubjectInfoRootNode = projectXMLRoot.xpath("/XCEDE/subject/subjectInfo[hash='" + curSubject + "']")
				# new subject, create a new ID for the subject
				#curSubjectID = nextSubjectID
				
				
			curSubjectURI = os.path.join(projectURI, str(curSubjectID).zfill(DataTransferConstants.padZerosLength))
			print "subject: " + str(curSubjectID)
			
			for curAcquisitionDate in subjects[curSubject].visits.iterkeys():
				print "\tvisit: " + curAcquisitionDate
				# check to see if the visit has already been added
				curVisitID = curAcquisitionDate
				curVisitNodeRoot = projectXMLRoot.xpath("/XCEDE/visit[@ID='" + curVisitID + "' and @subjectID='" + str(curSubjectID) + "' and @subjectGroupID='" + curSubjectType + "' and @projectID='" + projectName + "']")
				curVisitURI = os.path.join(curSubjectURI, curVisitID)
				if len(curVisitNodeRoot) == 0:
					curVisitNode = etree.SubElement(projectXMLRoot, "visit", \
					ID = curVisitID, \
					subjectID = str(curSubjectID), \
					subjectGroupID = curSubjectType, \
					subjectURI = curSubjectURI, \
					projectID = projectName, \
					projectURI = projectURI)
					curVisitInfoNode = etree.SubElement(curVisitNode, "visitInfo")
					etree.SubElement(curVisitInfoNode, "description")
					ageVisitNode = etree.SubElement(curVisitNode, "subjectAgeAtVisit")
					ageVisitNode.text = str(DataTransferConstants.getAgeAtVisit(curVisitID, subjects[curSubject].DOB))
					# put in a timestamp
					#curVisitInfoNode = etree.SubElement(curVisitNode, "visitInfo")
					#curTimeStampNode = etree.SubElement(curVisitInfoNode, "timeStamp")
					#curTimeStampNode.text = # work out the first and last time of scans on this date
				else:
					curVisitNode = curVisitNodeRoot[0]
				del curVisitNodeRoot
				
				curStudyURI = os.path.join(curVisitURI, curStudyID)
				curStudyNodeRoot = projectXMLRoot.xpath("/XCEDE/study[@ID='" + curStudyID + "' and @visitID='" + curVisitID + "' and @subjectID='" + str(curSubjectID) + "' and @subjectGroupID='" + curSubjectType + "' and @projectID='" + projectName + "']")
				
				if len(curStudyNodeRoot) == 0:
					studyNode = etree.SubElement(projectXMLRoot, "study", \
					ID = curStudyID, \
					projectID = projectName, \
					projectURI = projectURI, \
					subjectID = str(curSubjectID), \
					subjectGroupID = curSubjectType, \
					subjectURI = curSubjectURI, \
					visitID = str(curVisitID), \
					visitURI = curVisitURI)

					studyInfoNode = etree.SubElement(studyNode, "studyInfo")
					etree.SubElement(studyInfoNode, 'description')
					invoiceNode = etree.SubElement(studyNode, "invoiceNumber")
					invoiceNode.text = "Please edit"
					reportReceivedNode = etree.SubElement(studyNode, "reportReceived")
					reportReceivedNode.text = "no"
					curFollowupNode = etree.SubElement(studyNode, 'followUpRequired')
					curFollowupNode.text = "Please edit"
					curFollowupCompletedNode = etree.SubElement(studyNode, 'followUpCompleted')
					curFollowupCompletedNode.text = "Please edit"
				else:
					studyNode = curStudyNodeRoot[0]
				del curStudyNodeRoot

				# add the visit
				# sort the series by acquisition time
				SortedSeries = sorted(subjects[curSubject].visits[curAcquisitionDate], key = operator.itemgetter(1));

				for curSeries in SortedSeries:#subjects[curSubject].visits[curAcquisitionDate].iterkeys():
					# verify that we have image numbers 1 ... n
					
					episodeAdded = False
					# now create the episode, each series has its own episode
					#print curSubject + " " + curAcquisitionDate + " " + curDirectory + " " + curSeries + " " + str(numImages) + " images"
					for curImageType in subjects[curSubject].visits[curAcquisitionDate][curSeries].iterkeys():
						numImages = len(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType])
						mosaicFormat = False
						DTIImage = False
						BValues = []
						BVectors = []
						#for curImage in sorted(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType].iterkeys()):
						#	curFile = subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType][curImage]
						#	print 'image number: ' + str(DICOMTags[curFile]['imageNumber'])
						curImageIdx = 1
						sliceOrdering = 'unknown'
						for curImage in sorted(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType].iterkeys()):
							curFile = subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType][curImage]
							#etails = getDICOMKeyDCKEY.getDICOMMultipleKeyDCKEY(["0x0028,0x0010", "0x0028,0x0011", "0x0028,0x0100", "0x0028,0x0002", "0x0008,0x0032", "0x0008,0x0060", "0x0008,0x0070"], curFile)
							#DICOMTags[curFile]['acquisitionModality'] = details[5]
							if DICOMTags[curFile]['scannerManufacturer'] == None:
								print curFile + " no scanner manufacturer, guessing from institution name"
								
								if DICOMTags[curFile]['institutionName'] == None:
									print "No institution name either, unknown scanner manufacturer"
								else:
									# if the instutition has "children" in it, assume that it is the royal children's in which case they use siemens scanners
									if DICOMTags[curFile]['institutionName'].lower().find('children') != -1:
										DICOMTags[curFile]['scannerManufacturer'] = "SIEMENS"
									else:
										print "Could not guess scanner manufacturer from institution name"
							else:
								DICOMTags[curFile]['scannerManufacturer'] = DICOMTags[curFile]['scannerManufacturer'].upper()
							
							if episodeAdded == False:
								#DICOMTags[curFile]['seriesDescription'] = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0008", "0x103e", curFile)
								# deblank and replace reserved characters in unix
								DICOMTags[curFile]['seriesDescription'] = DICOMTags[curFile]['seriesDescription'].replace(" ", "")
								DICOMTags[curFile]['seriesDescription'] = DICOMTags[curFile]['seriesDescription'].replace("/", "_")
								DICOMTags[curFile]['seriesDescription'] = DICOMTags[curFile]['seriesDescription'].replace("<", "_")
								DICOMTags[curFile]['seriesDescription'] = DICOMTags[curFile]['seriesDescription'].replace(">", "_")
								DICOMTags[curFile]['seriesDescription'] = DICOMTags[curFile]['seriesDescription'].replace("(", "_")
								DICOMTags[curFile]['seriesDescription'] = DICOMTags[curFile]['seriesDescription'].replace(")", "_")
								curEpisodeID = DICOMTags[curFile]['seriesDescription']
								#print DICOMTags[curFile]['seriesNumber'] + " " + DICOMTags[curFile]['seriesDescription']
								# search for this Episode in the other images in this subject and visit, if we find another copy of this ID, then change the ID to something unique
								# the convention for this will be to use numbers appended onto the episode IDs
								# if we see gre_field_mapping already in the database
								# then the episodeID of the new episode will be gre-field-mapping-1
								# then the episodeID of the next new episode will be gre-field-mapping-2
								curEpisodeNodes = projectXMLRoot.xpath("//episode[@projectID='" + projectName + \
								"' and @subjectID='" + str(curSubjectID) + \
								"' and @subjectGroupID='" + curSubjectType + \
								"' and @visitID='" + str(curVisitID) + \
								"' and @studyID='" + str(curStudyID) + \
								"' and @ID='" + curEpisodeID + "']")
								curEpisodeURI = os.path.join(curStudyURI, curEpisodeID)
								if len(curEpisodeNodes) > 0:
									curEpisodeNode = curEpisodeNodes[0]
									#existingIDCount = 0
									#for curEpisodeNode in curEpisodeNodes:
									#	if curEpisodeNode.get("ID").find(curEpisodeID) != -1:
									#		existingIDCount = existingIDCount + 1
									#if existingIDCount > 0:
									#	curEpisodeID = curEpisodeID + "-" + str(existingIDCount)
									episodeAdded = True
								else:
									curEpisodeNode = etree.SubElement(projectXMLRoot, "episode", ID = curEpisodeID, projectID = projectName, projectURI = projectURI, \
									subjectID = str(curSubjectID), subjectGroupID = curSubjectType, subjectURI = curSubjectURI, visitID = str(curVisitID), \
									visitURI = curVisitURI, studyID = str(curStudyID), studyURI = curStudyURI)
									episodeAdded = True
									curEpisodeInfoNode = etree.SubElement(curEpisodeNode, "episodeInfo")
									etree.SubElement(curEpisodeInfoNode, "description")
								
							#numRows = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0028", "0x0010", curFile)
							#print str(type(DICOMTags[curFile]['numRows'])) + " " + curFile
							#numRows = int(DICOMTags[curFile]['numRows'], 16)
							totalNumRows = DICOMTags[curFile]['numRows']
							totalNumCols = DICOMTags[curFile]['numCols']
							#DICOMTags[curFile]['bitsAllocated'] = details[2]
							#DICOMTags[curFile]['bitsAllocated'] = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0028", "0x0100", curFile)
							#DICOMTags[curFile]['bitsAllocated'] = DICOMTags[curFile]['bitsAllocated']
							#samplesPerPixel = details[3]
							#samplesPerPixel = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0028", "0x0002", curFile)
							samplesPerPixel = DICOMTags[curFile]['samplesPerPixel']
							if DICOMTags[curFile]['seriesTime'] != None:
								seriesTime = float(DICOMTags[curFile]['seriesTime'])
							else:
								seriesTime = 0
							if DICOMTags[curFile]['acquisitionTime'] != None:
								acquisitionTime = float(DICOMTags[curFile]['acquisitionTime'])
							else:
								acquisitionTime = 0
							
							imageSize = totalNumRows * totalNumCols * DICOMTags[curFile]['bitsAllocated'] / 8 * samplesPerPixel
							imageOffset = os.path.getsize(curFile) - imageSize
							
							if curImageIdx == 1:
								
								#print curImageType + " " + str(curSeries)
								# work out the format and dimensions of the image and add it to the database
								# the major part of the work goes here
								#details = getDICOMKeyDCKEY.getDICOMMultipleKeyDCKEY(["0x0008,0x0008", "0x0020,0x0032"], curFile)
								firstSlicePosition = DICOMTags[curFile]['slicePosition']
								
								# check to see if the image is a "SECONDARY" or "DERIVED" image, these are not stored as raw data
								DimensionlessData = False

								curT = DICOMTags[curFile]['imageType'].split("\\")
								for curIdx in curT:
									if curIdx == "SECONDARY" or curIdx == "DERIVED":
										DimensionlessData = True
								if DimensionlessData == False:
									# get the slice orientation
									if DICOMTags[curFile]['sliceOrientation'] != None:
										#sliceOrientationElements = DICOMTags[curFile]['sliceOrientation'].split("\\")
										#sliceOrientationX = sliceOrientationElements[:3]
										#sliceOrientationY = sliceOrientationElements[-3:]
										sliceOrientationX = DICOMTags[curFile]['sliceOrientation'][0:3]
										sliceOrientationY = DICOMTags[curFile]['sliceOrientation'][3:6]
										for curElementX in range(len(sliceOrientationX)):
											sliceOrientationX[curElementX] = float(sliceOrientationX[curElementX])
										for curElementY in range(len(sliceOrientationY)):
											sliceOrientationY[curElementY] = float(sliceOrientationY[curElementY])

										sliceOrientationZ = []
										sliceOrientationZ.append(sliceOrientationX[1] * sliceOrientationY[2] - sliceOrientationX[2] * sliceOrientationY[1])
										sliceOrientationZ.append(sliceOrientationX[2] * sliceOrientationY[0] - sliceOrientationX[0] * sliceOrientationY[2])
										sliceOrientationZ.append(sliceOrientationX[0] * sliceOrientationY[1] - sliceOrientationX[1] * sliceOrientationY[0])
									else:
										sliceOrientationX = [1, 0, 0]
										sliceOrientationY = [0, 1, 0]
										sliceOrientationZ = [0, 0, 1]

# find out the number of slices and the dimensions
								if DICOMTags[curFile]['imageType'].find("MOSAIC") != -1 and DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':
									mosaicFormat = True
									numVolumes = numImages
									# this is the volume that we take the second time point from
									# since this is a mosaic format file, the time spacing is the time between the first and second volumes
									secondTimePointVolume = 2
									# for the siemens mosaic images, we need to extract the number of rows and columns
									# by dividing the FOV by the readout and phase FOVs
									# these are available as text fields in the DICOM header
									# they are read by getting the output of the 'strings' command on the DICOM file
									#curPipe = os.popen("strings \"" + curFile + "\"", 'r')
									#DICOMStrings = curPipe.readlines()
									#curPipe.close()
									fp = file(curFile, 'r')
									readFOV = None
									phaseFOV = None
									countedSlices = 0
									for curLine in fp.xreadlines():
										curLine = curLine.strip()
										if curLine.find("sSliceArray.ucMode") != -1:
											pat = re.compile("sSliceArray\.ucMode\s+=\s+(0x[0-9a-fA-F]+)");
											mat = pat.match(curLine)
											if mat != None:
												sliceOrdering = int(mat.group(1), 16)
												if sliceOrdering == 1:
													sliceOrdering = "ascending"
												elif sliceOrdering == 2:
													sliceOrdering = "descending"
												elif sliceOrdering == 4:
													sliceOrdering = "interleaved"
												else:
													sliceOrdering = "unknown"
											else:
												sliceOrdering = "unknown"

										if curLine.find("sSliceArray.asSlice[0].dPhaseFOV") != -1:
											pat = re.compile("sSliceArray\.asSlice\[0\]\.dPhaseFOV\s+=\s+(\d+)");
											mat = pat.match(curLine)
											if mat != None:
												phaseFOV = int(mat.group(1))
											else:
												#print "\"strings\" has stuffed up, waiting for proper line"
												pass
										#if curLine.find("sSliceArray.asSlice[0].dReadFOV") != -1:
										#if curLine.find("sSliceArray.asSlice[0].dPhaseFOV") != -1:
										if curLine.find("sSliceArray.asSlice[0].dReadoutFOV") != -1:
											pat = re.compile("sSliceArray\.asSlice\[0\]\.dReadoutFOV\s+=\s+(\d+)");
											mat = pat.match(curLine)
											if mat != None:
												readoutFOV = int(mat.group(1))
											else:
												pass
												#print "\"strings\" has stuffed up, waiting for proper line"
										#if curLine.find("sSliceArray.asSlice[0].dReadFOV") != -1:
										if DICOMTags[curFile]['FOV'] == None:
											if re.match("sSliceArray.asSlice\[\d+\].dReadoutFOV", curLine) != None:
												countedSlices = countedSlices + 1
										#if readFOV != None and phaseFOV != None:
										#	break
									#OV = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0051", "0x100c", curFile)
									fp.close()
									#print " numRows: " + str(numRows) + ", numCols: " + str(numCols) + " " + curFile
									# if the FOV is missing then we will work out how many slices there are by counting the number of lines that contain sSliceArray.asSlice[\d+].dReadoutFOV in the file
									# then the number of mosaic rows and columns will be the square root of this number
									if DICOMTags[curFile]['FOV'] == None:
										mosaicRows = int(math.ceil(math.sqrt(countedSlices)))
										mosaicCols = mosaicRows
										if mosaicRows != 0:
											numRows = totalNumRows / mosaicRows
											numCols = totalNumCols / mosaicCols
										else:
											# the SIEMENS ASCDATA IS MISSING
											# use acquisition matrix
											acqMatrixNumRows = max(DICOMTags[curFile]['acquisitionMatrix'][0], DICOMTags[curFile]['acquisitionMatrix'][2])
											acqMatrixNumCols = max(DICOMTags[curFile]['acquisitionMatrix'][1], DICOMTags[curFile]['acquisitionMatrix'][3])
											if totalNumRows % acqMatrixNumRows == 0 and totalNumCols % acqMatrixNumCols == 0:
												mosaicRows = totalNumRows / acqMatrixNumRows
												mosaicCols = totalNumCols / acqMatrixNumCols
											numRows = acqMatrixNumRows
											numCols = acqMatrixNumCols
									else:
										mat = re.match("FoV (\d+)\*(\d+)", DICOMTags[curFile]['FOV'])

										#print curFile + ", FOV: " + DICOMTags[curFile]['FOV']
										if int(mat.group(1)) % phaseFOV == 0 and int(mat.group(2)) % readoutFOV == 0:
											mosaicRows = int(mat.group(1)) / phaseFOV
											mosaicCols = int(mat.group(2)) / readoutFOV
											numRows = totalNumRows / mosaicRows
											numCols = totalNumCols / mosaicCols
										else:
											# check to see if we have some close rounding 
											mosaicRows = int(round(float(mat.group(1)) / float(phaseFOV)))
											mosaicCols = int(round(float(mat.group(2)) / float(readoutFOV)))

											#print "mosaic rounded: " + str(mosaicRows) + " " + str(mosaicCols)
											# test to see if the data size matches this mosaic size, if it does then we are in business
											if mosaicRows == 0:
												mosaicRows = 1
											if mosaicCols == 0:
												mosaicCols = 1

											numRows = totalNumRows / mosaicRows
											numCols = totalNumCols / mosaicCols

											numBytes = numRows * numCols * mosaicRows * mosaicCols * DICOMTags[curFile]['bitsAllocated'] / 8 * samplesPerPixel
											#print "num bytes: " + str(numBytes) + ", image size: " + str(imageSize)
											# if the number of bytes taken by the rounded mosaic is equivalent to the actual image size then we are in business
											if numBytes != imageSize:
												# something has not worked
												# print a warning message and assume that there is only one mosaic row and one mosaic column
												mosaicRows = 1
												mosaicCols = 1
												numRows = totalNumRows / mosaicRows
												numCols = totalNumCols / mosaicCols
										#print str(mosaicRows) + " " + str(mosaicCols)
										#print str(numRows) + " " + str(numCols)
									#if DICOMTags[curFile]['FOV'] == None:
									#print "mosaicRows: " + str(mosaicRows) + ", mosaicCols: " + str(mosaicCols) + ", numRows: " + str(numRows) + ", numCols: " + str(numCols) + " " + curFile

									if DICOMTags[curFile]['numSlices'] == None:
										if countedSlices == 0:
											numSlices = mosaicRows * mosaicCols # we have no way of knowing how many slices, since the header has been stripped of required fields
										else:
											numSlices = countedSlices

									else:
										numSlices = DICOMTags[curFile]['numSlices']
										#hexMatch = re.match('^0x[0-9a-fA-F]+$', DICOMTags[curFile]['numSlices'])
										#decMatch = re.match('^\d+$', DICOMTags[curFile]['numSlices'])
										#if hexMatch != None:
										#	numSlices = int(DICOMTags[curFile]['numSlices'], 16)
										#elif decMatch != None:
										#	numSlices = int(DICOMTags[curFile]['numSlices'])
										#else:
										#	print "Dont know how to convert numSlices: " + DICOMTags[curFile]['numSlices'] + " in " + curFile
										#$if type(DICOMTags[curFile]['numSlices']) is int:
										#else:
										#	numSlices = countedSlices
										#print "loading"
										#print numSlices
										#print type(numSlices)
										#print curFile
								else:
									mosaicFormat = False
									numSlices = numImages
									numVolumes = 1
									numRows = totalNumRows
									numCols = totalNumCols
								#if DICOMTags[curFile]['imageType'].find("MOSAIC") != -1 and DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':

								# SIEMENS old format, not mosaic but potentially multi-volume
								# so the number of slices is "countedSlices" that we computed earlier
								if mosaicFormat == False and DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':
									fp = file(curFile, 'r')
									countedSlices = 0
									for curLine in fp.xreadlines():
										curLine = curLine.strip()
										if re.match("^sSliceArray.asSlice\[\d+\].dReadoutFOV\s+=\s+\d+$", curLine) != None:
											countedSlices = countedSlices + 1
										#if readFOV != None and phaseFOV != None:
										#	break
									#OV = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0051", "0x100c", curFile)
									fp.close()

									numSlices = countedSlices
									if numSlices == 0:
										numVolumes = 1
										numSlices = numImages
										#print curFile + " " + str(DICOMTags[curFile]['seriesNumber']) + " " + str(numImages)
									else:
										numVolumes = numImages / numSlices
									# compute the time spacing by getting the acquisition time of the same slice in the next volume
									# non-mosaic format, the time spacing is the difference between the same slice in the next volume
									secondTimePointVolume = numSlices + 1

								if DimensionlessData == False:
									#pixelSpacing = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0028", "0x0030", curFile)
									if DICOMTags[curFile]['pixelSpacing'] == None:
										print curFile + ": pixelSpacing missing"
										xSpacing = 1
										ySpacing = 1
									else:
										xSpacing = DICOMTags[curFile]['pixelSpacing'][0]
										ySpacing = DICOMTags[curFile]['pixelSpacing'][1]
										#(xSpacing, ySpacing) = DICOMTags[curFile]['pixelSpacing'].split('\\')
									#zSpacing = getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0018", "0x0050", curFile)
										
								# find out the modality and then for each modality check the particular fields to find out the dimensions and datatype of the volumes
							#if curImageIdx == 1:
							if len(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType]) == 1 or curImageIdx == 2:#DICOMTags[curFile]['imageNumber'] == 2:
								#print "SliceX:"
								#print sliceOrientationX
								#print "SliceY:"
								#print sliceOrientationY
								#print "SliceZ:"
								#print sliceOrientationZ
								
								# LOOK FOR MOSAIC images
								if DICOMTags[curFile]['acquisitionModality'] == 'MR':
									pass
									#print "1"
								elif DICOMTags[curFile]['acquisitionModality'] == 'CT':
									#print DICOMTags[curFile]['seriesDescription']
									if DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':
										if mosaicFormat == True:
											print "Siemens mosaic for CT data not recognised"
										# we have all the info we need now

									elif DICOMTags[curFile]['scannerManufacturer'] == 'GE':
										print "4"
								# put in the MR or CT specific fields into the episode key
									
								# work out the number of acquisitions for this type of data file, find out how many acquisitions have occurred under this series name, subject and visit
								# here we also have to check see if the same acquisition time has been used
								# we therefore have a duplicate that shouldnt be inserted
								curAcquisitions = projectXMLRoot.xpath("/XCEDE/acquisition[@episodeID='" + curEpisodeID + \
								"' and @projectID='" + projectName + \
								"' and @subjectID='" + str(curSubjectID) + \
								"' and @visitID='" + curVisitID + \
								"' and @studyID='" + curStudyID + "']")
								duplicateAcquisitionFound = False
								maxAcquisitionID = -1
								for curAcquisitionIdx in range(len(curAcquisitions)):
									if int(curAcquisitions[curAcquisitionIdx].get("ID")) >= maxAcquisitionID:
										maxAcquisitionID = int(curAcquisitions[curAcquisitionIdx].get("ID")) + 1

									curDataResource = projectXMLRoot.xpath("/XCEDE/dataResource[@acquisitionID='" + curAcquisitions[curAcquisitionIdx].get("ID") + \
										"' and @episodeID='" + curEpisodeID + \
										"' and @projectID='" + projectName + \
										"' and @subjectID='" + str(curSubjectID) + \
										"' and @visitID='" + curVisitID + \
										"' and @studyID='" + curStudyID + "']")

									for otherAcqTimeNode in curAcquisitions[curAcquisitionIdx].iterchildren(tag = "acquisitionTime"):
										otherAcqTime = float(otherAcqTimeNode.text)
									
									otherEchoNumber = None
									for otherAcqEchoNumberNode in curAcquisitions[curAcquisitionIdx].iterchildren(tag = "echoNumber"):
										otherEchoNumber = int(otherAcqEchoNumberNode.text)
									if otherEchoNumber == None:
										#print "otherEchoNumber == None"
										otherEchoNumber = 1

									if len(curDataResource) == 1:
										curDataResource = curDataResource[0]
										if otherAcqTime == seriesTime and DICOMTags[curFile]['imageType'] == curDataResource.get("ID") and DICOMTags[curFile]['echoNumber'] == otherEchoNumber:
											duplicateAcquisitionFound = True
									else:
										print "warning: cannot do duplicate check since we have a missing dataResource object"
								if duplicateAcquisitionFound == True:
									print curEpisodeID + ", " + str(curSubjectID) + ", " + curVisitID + ", " + curStudyID + " duplicated data found, not importing"
									filesProcessed[curFile] = True
									break

								# increment that number by one
								#curAcquisitionID = len(curAcquisitions) + 1
								if maxAcquisitionID == -1:
									maxAcquisitionID = 1
								curAcquisitionID = str(maxAcquisitionID)

								#print curAcquisitionID
								curAcquisitionURI = os.path.join(curEpisodeURI, str(curAcquisitionID).zfill(DataTransferConstants.padZerosLength))
								acquisitionNode = etree.SubElement(projectXMLRoot, "acquisition", \
								ID = curAcquisitionID, \
								episodeID = curEpisodeID, \
								episodeURI = curEpisodeURI, \
								projectID = projectName, \
								projectURI = projectURI, \
								subjectID = str(curSubjectID), \
								subjectGroupID = curSubjectType, \
								subjectURI = curSubjectURI, \
								visitID = str(curVisitID), \
								visitURI = str(curVisitURI), \
								studyID = str(curStudyID), \
								studyURI = str(curStudyURI))
								acquisitionInfoNode = etree.SubElement(acquisitionNode, "acquisitionInfo")
								etree.SubElement(acquisitionInfoNode, "description")
								# the acquisition node allows wildcard attributes and children so we will put in details like TE, TR, Sequence name etc. here
								# this needs to go in the acqusition field since the episode can incorporate
								if DICOMTags[curFile]['acquisitionModality'] == 'MR':
									#details = getDICOMKeyDCKEY.getDICOMMultipleKeyDCKEY(["0x0018,0x0080", "0x0018,0x0081", "0x0018,0x0087", "0x0018,0x1314", "0x0018,0x0083"], curFile)
									
									if DICOMTags[curFile]['TR'] == None:
										TR = ""
									else:
										TR = float(DICOMTags[curFile]['TR'])

									if DICOMTags[curFile]['TE'] == None:
										TE = ""
									else:
										TE = float(DICOMTags[curFile]['TE'])

									if DICOMTags[curFile]['MRFieldStrength'] == None:
										MRFieldStrength = ""
									else:
										MRFieldStrength = float(DICOMTags[curFile]['MRFieldStrength'])

									if DICOMTags[curFile]['flipAngle'] == None:
										flipAngle = ""
									else:
										flipAngle = float(DICOMTags[curFile]['flipAngle'])
									
									if DICOMTags[curFile]['numAverages'] == None:
										numAverages = ""
									else:
										numAverages = float(DICOMTags[curFile]['numAverages'])

									etree.SubElement(acquisitionNode, "TR").text = str(TR)
									etree.SubElement(acquisitionNode, "TE").text = str(TE)
									etree.SubElement(acquisitionNode, "fieldStrength").text = str(MRFieldStrength)
									etree.SubElement(acquisitionNode, "flipAngle").text = str(flipAngle)
									etree.SubElement(acquisitionNode, "numAverages").text = str(numAverages)
									if DICOMTags[curFile]['echoNumber'] == None:
										etree.SubElement(acquisitionNode, "echoNumber").text = str(1)
									else:
										etree.SubElement(acquisitionNode, "echoNumber").text = str(DICOMTags[curFile]['echoNumber'])
									etree.SubElement(acquisitionNode, "scannerManufacturer").text = DICOMTags[curFile]['scannerManufacturer']
									etree.SubElement(acquisitionNode, "sliceOrdering").text = sliceOrdering
								#elif DICOMTags[curFile]['acquisitionModality'] == 'CT':
								etree.SubElement(acquisitionNode, "acquisitionTime").text = DICOMTags[curFile]['seriesTime']
								# now put the data in 
								if DimensionlessData == True:
									dataRecordType = "binaryDataResource_t"
								else:
									dataRecordType = "mappedBinaryDataResource_t"

								dataRootNode = etree.SubElement(projectXMLRoot, "dataResource", \
								name = os.path.join(curAcquisitionURI, "dicomfiles.tar.gz"), \
								type = dataRecordType, \
								ID = DICOMTags[curFile]['imageType'], \
								acquisitionID = str(curAcquisitionID), \
								acquisitionURI = curAcquisitionURI, \
								episodeID = curEpisodeID, \
								episodeURI = curEpisodeURI, \
								projectID = projectName, \
								projectURI = projectURI, \
								subjectID = str(curSubjectID), \
								subjectGroupID = curSubjectType, \
								subjectURI = str(curSubjectURI), \
								visitID = str(curVisitID), \
								visitURI = str(curVisitURI), \
								studyID = str(curStudyID), \
								studyURI = str(curStudyURI), \
								format='DICOM')
								if DICOMTags[curFile]['bitsAllocated'] == 8:
									dataType = "uint8"
								elif DICOMTags[curFile]['bitsAllocated'] == 16:
									dataType = "int16"
								else:
									dataType = 'unknown'

								dataTypeNode = etree.SubElement(dataRootNode, 'elementType')
								dataTypeNode.text = dataType
								byteOrderNode = etree.SubElement(dataRootNode, 'byteOrder')
								byteOrderNode.text = 'lsbfirst'
								if DimensionlessData == False:
									#print str(numRows) + " " + str(numCols) + " " + str(numSlices) + " " + str(numVolumes)
									xDimensionNode = etree.SubElement(dataRootNode, "dimension", label="x")
									xSizeNode = etree.SubElement(xDimensionNode, "size")
									xSizeNode.text = str(numCols)
									xSpacingNode = etree.SubElement(xDimensionNode, "spacing")
									xSpacingNode.text = str(xSpacing)
									xGapNode = etree.SubElement(xDimensionNode, "gap")
									xGapNode.text = str(0)
									xUnitsNode = etree.SubElement(xDimensionNode, "units")
									xUnitsNode.text = "mm"
									#print sliceOrientationX
									xDirectionNode = etree.SubElement(xDimensionNode, "direction")
									xDirectionNode.text = " ".join(str(v) for v in sliceOrientationX)
									yDimensionNode = etree.SubElement(dataRootNode, "dimension", label="y")
									ySizeNode = etree.SubElement(yDimensionNode, "size")
									ySizeNode.text = str(numRows)
									ySpacingNode = etree.SubElement(yDimensionNode, "spacing")
									ySpacingNode.text = str(ySpacing)
									yGapNode = etree.SubElement(yDimensionNode, "gap")
									yGapNode.text = str(0)
									yUnitsNode = etree.SubElement(yDimensionNode, "units")
									yUnitsNode.text = "mm"
									yDirectionNode = etree.SubElement(yDimensionNode, "direction")
									yDirectionNode.text = " ".join(str(v) for v in sliceOrientationY)
									#print type(curSubject)
									#print type(curAcquisitionDate)
									#print type(curSeries)
									#print type(curImageType)
									#print curImageType
									if len(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType]) > 1:
										# this executes only on the second slice, computes the slice spacing
										secondSlicePosition = DICOMTags[curFile]['slicePosition']#$getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0020", "0x0032", curFile)
										if secondSlicePosition != None:
											#firstSlicePosition = firstSlicePosition.split('\\')
											#secondSlicePosition = secondSlicePosition.split('\\')
											#print curFile
											#print " ".join(firstSlicePosition)
											#print " ".join(secondSlicePosition)

											zSpacing = math.sqrt(\
											(float(firstSlicePosition[0]) - float(secondSlicePosition[0])) * (float(firstSlicePosition[0]) - float(secondSlicePosition[0])) + \
											(float(firstSlicePosition[1]) - float(secondSlicePosition[1])) * (float(firstSlicePosition[1]) - float(secondSlicePosition[1])) + \
											(float(firstSlicePosition[2]) - float(secondSlicePosition[2])) * (float(firstSlicePosition[2]) - float(secondSlicePosition[2])))
											# if we get a slice thickness of zero, try the spacing between slices key
											if zSpacing == 0.0:
												zSpacing = DICOMTags[curFile]['spacingBetweenSlices']
												if zSpacing == None:
													# then try slice thickness key
													zSpacing = DICOMTags[curFile]['sliceThickness']#getDICOMKeyDCKEY.getDICOMKeyDCKEY("0x0018", "0x0050", curFile)
													if zSpacing == None:
														print curFile + ": zSpacing really missing"
														zSpacing = 1
													else:
														zSpacing = float(zSpacing)		
												else:
													zSpacing = float(zSpacing)		
										else:
											print curFile + ": zSpacing really missing"
											zSpacing = 1
									else:
										print curFile + ": zSpacing really missing"
										zSpacing = 1
		
									if mosaicFormat == True:
										zDimensionNode = etree.SubElement(dataRootNode, "dimension", label="z", splitRank="1")
										zSizeNode = etree.SubElement(zDimensionNode, "size")
										zSizeNode.text = str(mosaicRows)
										zSpacingNode = etree.SubElement(zDimensionNode, "spacing")
										zSpacingNode.text = str(zSpacing)
										zGapNode = etree.SubElement(zDimensionNode, "gap")
										zGapNode.text = str(0)
										zUnitsNode = etree.SubElement(zDimensionNode, "units")
										zUnitsNode.text = "mm"
										zDirectionNode = etree.SubElement(zDimensionNode, "direction")
										zDirectionNode.text = " ".join(str(v) for v in sliceOrientationZ)
										#print curFile
										#print numSlices
										#print type(numSlices)
										outputSelectString = " ".join(str(v) for v in range(int(numSlices)))
										
										zDimensionNode = etree.SubElement(dataRootNode, "dimension", label="z", splitRank="2", outputSelect = outputSelectString)
										zSizeNode = etree.SubElement(zDimensionNode, "size")
										zSizeNode.text = str(mosaicCols)
										zSpacingNode = etree.SubElement(zDimensionNode, "spacing")
										zSpacingNode.text = str(zSpacing)
										zGapNode = etree.SubElement(zDimensionNode, "gap")
										zGapNode.text = str(0)
										zUnitsNode = etree.SubElement(zDimensionNode, "units")
										zUnitsNode.text = "mm"
										zDirectionNode = etree.SubElement(zDimensionNode, "direction")
										zDirectionNode.text = " ".join(str(v) for v in sliceOrientationZ)
									else:
										zDimensionNode = etree.SubElement(dataRootNode, "dimension", label="z")
										zSizeNode = etree.SubElement(zDimensionNode, "size")
										zSizeNode.text = str(numSlices)
										zSpacingNode = etree.SubElement(zDimensionNode, "spacing")
										zSpacingNode.text = str(zSpacing)
										zGapNode = etree.SubElement(zDimensionNode, "gap")
										zGapNode.text = str(0)
										zUnitsNode = etree.SubElement(zDimensionNode, "units")
										zUnitsNode.text = "mm"
										zDirectionNode = etree.SubElement(zDimensionNode, "direction")
										zDirectionNode.text = " ".join(str(v) for v in sliceOrientationZ)
									#if mosaicFormat == True:
								#if DimensionlessData == False:
								#print "DimensionlessData: " + str(DimensionlessData)
								# calculate the directories for each subject, study, visit, episode, acqusition
							#if len(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType]) == 1 or curImageIdx == 2:#DICOMTags[curFile]['imageNumber'] == 2:
							if curImageIdx > 1 and DimensionlessData == False:
								if numVolumes > 1 and curImage == secondTimePointVolume:
									tSpacing = acquisitionTime - firstAcquisitionTime
									if tSpacing > 0:
										tDimensionNode = etree.SubElement(dataRootNode, "dimension", label="t")
										tSizeNode = etree.SubElement(tDimensionNode, "size")
										tSizeNode.text = str(numVolumes)
										tSpacingNode = etree.SubElement(tDimensionNode, "spacing")
										tSpacingNode.text = str(tSpacing)
										tGapNode = etree.SubElement(tDimensionNode, "gap")
										tGapNode.text = str(0)
										tUnitsNode = etree.SubElement(tDimensionNode, "units")
										tUnitsNode.text = "sec"
										tDataPointsNode = etree.SubElement(tDimensionNode, "datapoints")
										tDataPointsNode.text = " ".join(str(v) for v in numpy.arange(0, tSpacing * (numVolumes - 1), tSpacing))
									else:
										if mosaicFormat == False:
											# we made a mistake and the number of volumes is actually the number of slices
											zSizeNode.text = str(numVolumes)
										else:
											# mosaic format, something is wrong with the image header, so we actually have a timeseries volume
											# get the tSpacing from the TR if MR, otherwise just use 1
											if DICOMTags[curFile]['acquisitionModality'] == 'MR':
												tSpacing = DICOMTags[curFile]['TR'] / 1000
											else:
												tSpacing = 1
											tDimensionNode = etree.SubElement(dataRootNode, "dimension", label="t")
											tSizeNode = etree.SubElement(tDimensionNode, "size")
											tSizeNode.text = str(numVolumes)
											tSpacingNode = etree.SubElement(tDimensionNode, "spacing")
											tSpacingNode.text = str(tSpacing)
											tGapNode = etree.SubElement(tDimensionNode, "gap")
											tGapNode.text = str(0)
											tUnitsNode = etree.SubElement(tDimensionNode, "units")
											tUnitsNode.text = "sec"
											tDataPointsNode = etree.SubElement(tDimensionNode, "datapoints")
											tDataPointsNode.text = " ".join(str(v) for v in numpy.arange(0, tSpacing * (numVolumes - 1), tSpacing))

								#if numVolumes > 1 and curImage == secondTimePointVolume:
							#if curImageIdx > 1 and DimensionlessData == False:
							# mosaic: the entire volume is acquired in the one DICOM file, the second time point we need is in the second file
							# non-mosaic: the second time point is the acquisition time of the same slice in the second volume
							# check for MR - DTI - SIEMENS data
							if DICOMTags[curFile]['acquisitionModality'] == 'MR':
								if DICOMTags[curFile]['scannerManufacturer'] == 'SIEMENS':

									#details = getDICOMKeyDCKEY.getDICOMMultipleKeyDCKEY(["0x0019,0x100c", "0x0019,0x100e"], curFile)
									if DICOMTags[curFile]['BValue'] != None:
										BValues.append(int(DICOMTags[curFile]['BValue']))
										# correct BVector for slice orientation
										# X component is X slice direction dot BVector
										# Y component is Y slice direction dot BVector
										# Z component is Z slice direction dot BVector
										if DICOMTags[curFile]['BVector'] != None and int(DICOMTags[curFile]['BValue']) > 0:
											#curBVector = DICOMTags[curFile]['BVector'].split(',')
											curBVector = DICOMTags[curFile]['BVector']
											for idx in range(len(curBVector)):
												curBVector[idx] = float(curBVector[idx])
											curBVectorMag = 0.0
											for idx in range(len(curBVector)):
												curBVectorMag = curBVectorMag + curBVector[idx] * curBVector[idx]
											curBVectorMag = math.sqrt(curBVectorMag)
											for idx in range(len(curBVector)):
												curBVector[idx] = curBVector[idx] / curBVectorMag
											# this is for FSL, may transfer this to dt_checkout_volume
											correctedBVector = curBVector
											#correctedBVector[0] = curBVector[0] * sliceOrientationX[0] + curBVector[1] * sliceOrientationX[1] + curBVector[2] * sliceOrientationX[2]
											#correctedBVector[1] = curBVector[0] * sliceOrientationY[0] + curBVector[1] * sliceOrientationY[1] + curBVector[2] * sliceOrientationY[2]
											#correctedBVector[1] = -correctedBVector[1] # to be compatible with dcm2nii, haven't figure this one out yet
											#correctedBVector[2] = curBVector[0] * sliceOrientationZ[0] + curBVector[1] * sliceOrientationZ[1] + curBVector[2] * sliceOrientationZ[2]
											BVectors.append(correctedBVector)
											DTIImage = True
										else:
											BVectors.append([0, 0, 0])
									elif DICOMTags[curFile]['CSAImageHeader'] != None:
										# if this isnt a mosaic file, we only need to get the diffusion directions from the first slice in each volume
										if mosaicFormat == True or (curImageIdx % numSlices) == 1:
												# check for the diffusion information in the CSA Image Header
												if DICOMTags[curFile]['CSAHeaderFormat'] != None and DICOMTags[curFile]['CSAHeaderFormat'] == 'SIEMENS CSA HEADER':
													BVectorAndBValue = getBValueBVectorSiemensCSA(DICOMTags[curFile]['CSAImageHeader'], curFile)
													if BVectorAndBValue == None:
														print curFile
												else:
													BVectorAndBValue = {}
													BVectorAndBValue['BValue'] = 0
													BVectorAndBValue['BVector'] = [0, 0, 0]
												BValues.append(BVectorAndBValue['BValue'])
												BVectors.append(BVectorAndBValue['BVector'])
												if BVectorAndBValue['BValue'] != 0:
													DTIImage = True
													#print "Got DTI Data via CSA image header: " + str(BVectorAndBValue['BValue']) + ", [" + ", ".join(["%s" % (v) for v in BVectorAndBValue['BVector']]) + "]"
									else:
										BValues.append(0)
										BVectors.append([0, 0, 0])
							if curImageIdx == 1:#DICOMTags[curFile]['imageNumber'] == 1:
								firstImageSize = imageSize
								firstImageOffset = imageOffset
								firstFileName = curFile
								firstAcquisitionTime = acquisitionTime

							# we have seen the first file, so put the URI in if we only have one file or the current file is number 2
							if len(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType]) == 1 or curImageIdx == 2:#DICOMTags[curFile]['imageNumber'] == 2:
								curFileNode = etree.SubElement(dataRootNode, 'uri', size = str(firstImageSize), offset = str(firstImageOffset))
								(head, tail) = os.path.split(firstFileName)
								curFileNode.text = tail
							# put the current file's URI in
							if curImageIdx > 1:#DICOMTags[curFile]['imageNumber'] > 1:
								curFileNode = etree.SubElement(dataRootNode, 'uri', size = str(imageSize), offset = str(imageOffset))
								(head, tail) = os.path.split(curFile)
								curFileNode.text = tail
							curImageIdx = curImageIdx + 1
						#for curImage in sorted(subjects[curSubject].visits[curAcquisitionDate][curDirectory][curSeries].iterkeys()):
						
						if duplicateAcquisitionFound == False:
							# add in the BVectors and BValues if we have found a DTI volume
							if DTIImage == True:
								etree.SubElement(acquisitionNode, "BValues").text = ",".join(["%s" % (v) for v in BValues])
								for idx in range(len(BVectors)):
									BVectors[idx] = ",".join(["%s" % (v) for v in BVectors[idx]])
								etree.SubElement(acquisitionNode, "BVectors").text = ";".join(BVectors)

							#print subjects[curSubject].name + " " + curAcquisitionURI + " " + curImageType
							# create the directory for the series
							try:
								os.makedirs(DataTransferConstants.acquisitionFileDirectory(databaseDirectory, projectName, curSubjectID, curVisitID, curStudyID, curEpisodeID, curAcquisitionID))
								#print "making " + os.path.join(databaseDirectory, curAcquisitionURI)
							except os.error, e:
								if e.errno == errno.EEXIST:
									print "directory " + curAcquisitionURI + " already exists, overwriting files underneath"
								if e.errno != errno.EEXIST:
									raise

							# anonymise the DICOM files by putting them into the destination directory with dccp
							# anonymise the patient name and address fields by encrypting them with the public key for the project
							curSeriesFiles = []
							for curImage in sorted(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType].iterkeys()):
								curFile = subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType][curImage]
								(head, tail) = os.path.split(curFile)
								filesProcessed[curFile] = True

								#subjectNameEnc = DataTransferConstants.encryptTextWithProject(databaseDirectory, projectName, subjects[curSubject].name)
								#subjectAddressEnc = DataTransferConstants.encryptTextWithProject(databaseDirectory, projectName, subjects[curSubject].address)
								curSeriesFiles.append(tail)
								curSeriesFiles.append("." + tail)
								#commandString = DCCP + " -ra PatientAddress \"" + subjectAddressEnc + "\" -ra PatientName \"" + subjectNameEnc + "\" < \"" + curFile + "\" > \"" + os.path.join(databaseDirectory, curAcquisitionURI, tail) + "\" 2>/dev/null"
								#os.system(commandString)
								#pydicom method
								ds = dicom.filereader.read_file(curFile, stop_before_pixels=False)
								# new strategy for anonymisation
								# clear the address and name, write to another file
								fp = file(os.path.join(temporaryDirectory, "." + tail), 'w')

								fp.write(subjects[curSubject].nameEnc + "\n")
								fp.write(subjects[curSubject].addressEnc + "\n")
								fp.close()

								if not (0x10,0x10) in ds:
									ds.AddNew(0x00100010, "PN", "")
								else:
									ds[0x10,0x10].value = "" # subject name

								if not (0x10,0x1040) in ds:
									ds.AddNew(0x00101040, "LO", "")
								else:
									ds[0x10,0x1040].value = "" # subject address
								#dicom.filewriter.WriteFile(os.path.join(databaseDirectory, curAcquisitionURI, tail), ds)
								dicom.filewriter.write_file(os.path.join(temporaryDirectory, tail), ds)

								del ds
							print "\t\tseries " + str(curSeries) + " -> " + curAcquisitionURI
							#compressTarFiles = False
							TarListFile = tempfile.NamedTemporaryFile(mode = 'w+t')
							TarListFile.write("\n".join(["%s" % (v) for v in curSeriesFiles]))
							TarListFile.flush()

							commandString = "tar c -C \"" + temporaryDirectory + "\" -T " + TarListFile.name + " | " + compressionProgram + " -1 > \"" + \
							os.path.join(DataTransferConstants.acquisitionFileDirectory(databaseDirectory, projectName, curSubjectID, curVisitID, curStudyID, curEpisodeID, curAcquisitionID), "dicomfiles.tar.gz") + "\""
							os.system(commandString)
							TarListFile.close()
							
							for curImageFile in curSeriesFiles:
								os.remove(os.path.join(temporaryDirectory, curImageFile))
						else:
							# make all the files in the duplicated acquisition as processed so they dont get printed
							print "marking files in duplicated data as \"processed\""
							for curImage in sorted(subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType].iterkeys()):
								curFile = subjects[curSubject].visits[curAcquisitionDate][curSeries][curImageType][curImage]
								filesProcessed[curFile] = True
						#if duplicateAcquisitionFound == False:
					#for curImageType in subjects[curSubject].visits[curAcquisitionDate][curSeries].iterkeys())

				#for curSeries in subjects[curSubject].visits[curAcquisitionDate].iterkeys():
			#for curAcquisitionDate in subjects[curSubject].iterkeys():
		#for curSubject in subjects.iterkeys():
		# write the mappings from hashes to names, addresses, birthdates etc. to the mapping file
		#mappingFile = file(os.path.join(databaseDirectory, projectName, "namehashmapping.txt"), "a")
		
		#for curSubject in subjectsToAddToNameMapping:
		#	mappingFile.write(str(subjects[curSubject].ID) + "," + curSubject + "," + subjects[curSubject].name + "," + subjects[curSubject].sex + "," + subjects[curSubject].DOB + "," + subjects[curSubject].address + "\n")
		#mappingFile.close()
		nameHashMapping.write()

		userFields = DataTransferUserFields.UserFields(projectName=projectName, databaseDirectory=databaseDirectory)
		userFields.addFieldsInXML(projectXMLRoot = projectXMLRoot)

		projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print = True)
	#if XJOBS != None and GZIPMAX != None:
	#	command = subprocess.Popen(XJOBS + " " + GZIPMAX + " 2>&1 > /dev/null", shell = True, stdin = subprocess.PIPE)
	#	commandPipe = command.stdin
	#	commandPipe.write("\n".join(TarFilesToCompress))
	#	print "compressing files with xjobs"
	#	command.communicate()
		os.removedirs(temporaryDirectory)
		print "files not processed:"
		for curFile in sorted(filesProcessed.iterkeys()):
			if filesProcessed[curFile] == False:
				print curFile
	#if os.path.isdir(baseDirectory):
#def main():

def getBValueBVectorSiemensCSA(CSAImageHeader, curFile):	

	CSAEntries = ['EchoLinePosition',
			'EchoColumnPosition',
			'EchoPartitionPosition',
			'UsedChannelMask',
			'Actual3DImaPartNumber',
			'ICE_Dims',
			'B_value',
			'Filter1',
			'Filter2',
			'ProtocolSliceNumber',
			'RealDwellTime',
			'PixelFile',
			'PixelFileName',
			'SliceMeasurementDuration',
			'SequenceMask',
			'AcquisitionMatrixText',
			'MeasuredFourierLines',
			'FlowEncodingDirection',
			'FlowVenc',
			'PhaseEncodingDirectionPositive',
			'NumberOfImagesInMosaic',
			'DiffusionGradientDirection',
			'ImageGroup',
			'SliceNormalVector',
			'DiffusionDirectionality',
			'TimeAfterStart',
			'FlipAngle',
			'SequenceName',
			'RepetitionTime',
			'EchoTime',
			'NumberOfAverages',
			'VoxelThickness',
			'VoxelPhaseFOV',
			'VoxelReadoutFOV',
			'VoxelPositionSag',
			'VoxelPositionCor',
			'VoxelPositionTra',
			'VoxelNormalSag',
			'VoxelNormalCor',
			'VoxelNormalTra',
			'VoxelInPlaneRot',
			'ImagePositionPatient',
			'ImageOrientationPatient',
			'PixelSpacing',
			'SliceLocation',
			'SliceThickness',
			'SpectrumTextRegionLabel',
			'Comp_Algorithm',
			'Comp_Blended',
			'Comp_ManualAdjusted',
			'Comp_AutoParam',
			'Comp_AdjustedParam',
			'Comp_JobID',
			'FMRIStimulInfo',
			'FlowEncodingDirectionString',
			'RepetitionTimeEffective']

	fp = StringIO(CSAImageHeader)
	# tells us what format the CSA Header is in
	magicNumber = fp.read(4)
	if magicNumber == "SV10":
		# format 1
		unused = fp.read(4)
		numElements = unpack('I', fp.read(4))[0]
	else:
		numElements = unpack('I', fp.read(4))[0]
		unused = fp.read(4)

	if numElements < 0 or numElements > 128:
		print "numElements should be between 0 and 128 inclusive, returning blank"
		return None
	unused = unpack('I', fp.read(4))[0]
	if unused != 77:
		print "unused is not 77"
	if numElements > len(CSAEntries):
		numElements = len(CSAEntries)
	elements = []
	for curElement in range(numElements):

		curElementHash = {}
		curElementHash['name'] = fp.read(64).strip(" \x00")

		if curElementHash['name'].find(CSAEntries[curElement]) == 0:
			curElementHash['name'] = CSAEntries[curElement]
		else:
			print "name doesnt match dictionary in CSA Header"

		curElementHash['VM'] = unpack('I', fp.read(4))[0]
		curElementHash['VR'] = fp.read(4).strip()
		curElementHash['SyngoDT'] = unpack('I', fp.read(4))[0]
		curElementHash['numItems'] = unpack('I', fp.read(4))[0]
		curElementHash['XX'] = unpack('I', fp.read(4))[0]
		curElementHash['items'] = []
		for curItem in range(curElementHash['numItems']):
			curItemHash = {}
			curItemHash['XX'] = []
			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
			curItemHash['XX'].append(unpack('I', fp.read(4))[0])
			if magicNumber == "SV10":
				curLength = curItemHash['XX'][1]
			else:
				curLength = curItemHash['XX'][0] - curElementHash['numItems']

			curItemHash['val'] = fp.read(curLength).strip(" \x00")
			#i = curItemHash['val'].index('\x00')
			#curItemHash['val'] = curItemHash['val'][:i]
			# read some bytes to pad, since the the elements should be on 4-byte boundaries
			unused = fp.read((4 - (curLength % 4)) % 4)
			curElementHash['items'].append(curItemHash)
		#for curItem in range(numItems):
		elements.append(curElementHash)
	#for curElement in range(numElements):
	BStats = {}
	BStats['BVector'] = [0, 0, 0]
	BStats['BValue'] = 0
	for curElement in range(len(elements)):
		#print elements[curElement]['name']
		if elements[curElement]['name'] == 'DiffusionGradientDirection':
			if elements[curElement]['numItems'] == 6:
				curBVector = []
				curBVector.append(float(elements[curElement]['items'][0]['val'].strip()))
				curBVector.append(float(elements[curElement]['items'][1]['val'].strip()))
				curBVector.append(float(elements[curElement]['items'][2]['val'].strip()))
			else:
				#print str(elements[curElement]['numItems'])
				curBVector = [0, 0, 0]
			BStats['BVector'] = curBVector

		if elements[curElement]['name'] == 'B_value':
			if elements[curElement]['numItems'] > 0:
				BStats['BValue'] = int(elements[curElement]['items'][0]['val'].strip())
			else:
				#print "numItems == 0"
				BStats['BValue'] = 0
	#print BStats['BValue']
	#print BStats['BVector']
	return BStats

if __name__ == "__main__":
	main()
