#!/usr/bin/python

from lxml import etree

import DataTransferConstants

# python standard library
import re
import os
import errno
import sys
import getopt
import subprocess
import hashlib
import math
import Numeric
import tempfile

# pydicom
import dicom

def usage():
	print "Usage: " + sys.argv[0] + " -p <project name> [--database-directory=<>] --subject-id=<> --new-subject-id=<> [-v]"
	print
	print "Mandatory arguments:"
	print "-p <project name>: the project name, the project's directory must exist and be created by dt_create_project.py"
	print 
	print "--subject-id=<> the numeric id of a subject"
	print "--new-subject-id=<>, matched visits will have their dates changed to whatever is specified here"
	print
	print "Optional arguments:"
	print "-h: help (this message)"
	print "-v: verbose, prints out when directories are moved"
	print "--database-directory=<database directory>: the root directory of where the database is situated"

import shutil

# moves a subject's data into another subject type/subject/study/
# rationale:
# Spelling errors in names or errors in birthdates may result in either a new subject being created when the data is already there
# or the user may just wish to change the subject type from patient to control

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:hv", ["database-directory=", "subject-id=", "new-subject-id="])

	projectName = None
	databaseDirectory = None
	baseDirectory = None
	oldSubjectID = None
	newSubjectID = None
	verbose = False

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '-v':
			verbose = True
		if o == '-h':
			usage()
			quit()
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--subject-id':
			oldSubjectID = a
		if o == '--new-subject-id':
			newSubjectID = a
	#for o, a in opts:
	
	temporaryDirectory = tempfile.mkdtemp()

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None or oldSubjectID == None or newSubjectID == None:
		print "Mandatory argument not given"
		usage()
		quit()
	
	projectName = projectName.strip('/')
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "project directory could not be found"
		quit()

	pat = re.compile('^\d+$')
	
	if re.match(pat, oldSubjectID) == None:
		print "subject id was not numeric"
		quit()
	
	if re.match(pat, newSubjectID) == False:
		print "new subject id was not numeric"
		quit()
	
	oldSubjectID = int(oldSubjectID)
	newSubjectID = int(newSubjectID)

	if oldSubjectID == newSubjectID:
		print "the old subject id equals the new subject id, nothing would happen"
		quit()
	
	projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
	projectXMLRoot = projectXML.getroot()
	
	encryptionObject = DataTransferConstants.encryptionModule(projectName=projectName, databaseDirectory=databaseDirectory, publicOrPrivate="private")

	nameHashMapping = DataTransferProject.NameHashMapping(databaseDirectory = databaseDirectory, projectName = projectName)
	
	projectPatientsListRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='patients']")
	if len(projectPatientsListRoot) == 0:
		print "No patients group list, should not happen"
		quit()
	else:
		projectPatientsListRoot = projectPatientsListRoot[0]

	projectControlsListRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='controls']")
	if len(projectControlsListRoot) == 0:
		print "No controls group list, should not happen"
		quit()
	else:
		projectControlsListRoot = projectControlsListRoot[0]

	# match all the DataResource nodes with the input pattern

	for curSubjectNode in projectControlsListRoot.iterchildren():
		if int(curSubjectNode.text) == oldSubjectID:
			curSubjectType = 'controls'
		if int(curSubjectNode.text) == newSubjectID:
			newSubjectType = 'controls'
	for curSubjectNode in projectPatientsListRoot.iterchildren():
		if int(curSubjectNode.text) == oldSubjectID:
			curSubjectType = 'patients'
		if int(curSubjectNode.text) == newSubjectID:
			newSubjectType = 'patients'

	subjectNode = projectXMLRoot.xpath("/XCEDE/subject[@ID='" + str(oldSubjectID) + "']")
	newSubjectNode = projectXMLRoot.xpath("/XCEDE/subject[@ID='" + str(newSubjectID) + "']")
	#quit()
	if len(subjectNode) == 0:
		print "The subject could not be found"
		quit()
	elif len(newSubjectNode) == 0:
		print "The new subject id could not be found in the database"
		quit()
	else:
		subjectNode = subjectNode[0]
		newSubjectNode = newSubjectNode[0]
		
		# compute an encrypted name and address for the new data
		subjectNameEnc = encryptionObject.encryptText(nameHashMapping[newSubjectID]['name'])
		subjectAddressEnc = encryptionObject.encryptText(nameHashMapping[newSubjectID]['address'])
		if verbose == True:
			print "reanonymizing files"
		#DataTransferRewritePrivate.rewritePrivateTags(projectDirectory=os.path.join(databaseDirectory, projectName), projectXMLRoot=projectXMLRoot, subjectNameEnc=subjectNameEnc, subjectAddressEnc=subjectAddressEnc, oldSubjectID=newSubjectID)

		existingVisitNodes = projectXMLRoot.xpath("/XCEDE/visit[@subjectID='" + newSubjectID + "']")
		visitNodes = projectXMLRoot.xpath("/XCEDE/visit[@subjectID='" + oldSubjectID + "']")

		for curVisitNode in visitNodes:
			visitID = curVisitNode.get("ID")
			curExistingVisitNode = None
			for curNode in existingVisitNodes:
				if curVisitNode.get("ID") == curNode.get("ID"):
					curExistingVisitNode = curNode
					break
				#if curVisitNode.get("ID") == curNode.get("ID"):
			#for curNode in existingVisitNodes:
			if curExistingVisitNode == None:
				#print "visit node not found"
				# the visit doesnt exist in the new subject, move the visit directory, change all the nodes in the XML
				studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + oldSubjectID + "']")
				episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + oldSubjectID + "']")
				acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + oldSubjectID + "']")
				dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + oldSubjectID + "']")
				
				curVisitNode.set("subjectID", newSubjectID)
				curVisitNode.set("subjectGroupID", newSubjectType)
				subjectURI = os.path.join(curVisitNode.get("projectID"), newSubjectID.zfill(DataTransferConstants.padZerosLength))
				visitURI = os.path.join(subjectURI, visitID)

				for curNodeToChange in studyNodes:
					curNodeToChange.set("subjectID", newSubjectID)
					curNodeToChange.set("subjectGroupID", newSubjectType)
					curNodeToChange.set("subjectURI", subjectURI)
					curNodeToChange.set("visitURI", visitURI)

				for curNodeToChange in episodeNodes:
					studyURI = os.path.join(visitURI, curNodeToChange.get("studyID"))
					curNodeToChange.set("subjectID", newSubjectID)
					curNodeToChange.set("subjectGroupID", newSubjectType)
					curNodeToChange.set("subjectURI", subjectURI)
					curNodeToChange.set("visitURI", visitURI)
					curNodeToChange.set("studyURI", studyURI)
				
				for curNodeToChange in acquisitionNodes:
					studyURI = os.path.join(visitURI, curNodeToChange.get("studyID"))
					episodeURI = os.path.join(studyURI, curNodeToChange.get("episodeID"))
					curNodeToChange.set("subjectID", newSubjectID)
					curNodeToChange.set("subjectGroupID", newSubjectType)
					curNodeToChange.set("visitURI", visitURI)
					curNodeToChange.set("studyURI", studyURI)
					curNodeToChange.set("episodeURI", episodeURI)
				
				for curNodeToChange in dataResourceNodes:
					studyURI = os.path.join(visitURI, curNodeToChange.get("studyID"))
					episodeURI = os.path.join(studyURI, curNodeToChange.get("episodeID"))
					acquisitionURI = os.path.join(episodeURI, curNodeToChange.get("acquisitionID"))
					curNodeToChange.set("subjectID", newSubjectID)
					curNodeToChange.set("subjectGroupID", newSubjectType)
					curNodeToChange.set("visitURI", visitURI)
					curNodeToChange.set("studyURI", studyURI)
					curNodeToChange.set("episodeURI", episodeURI)
					curNodeToChange.set("acquisitionURI", acquisitionURI)
				# move the study from the old visit to the new visit
				sourceDir = os.path.join(databaseDirectory, projectName, 'data', oldSubjectID.zfill(DataTransferConstants.padZerosLength), visitID)
				destDir = os.path.join(databaseDirectory, projectName, 'data', newSubjectID.zfill(DataTransferConstants.padZerosLength), visitID)
				if verbose == True:
					print "mv " + sourceDir + " -> " + destDir
				shutil.move(sourceDir, destDir)
			else: #if curExistingVisitNode == None:
				# the visit does exist so we have to check for studies that are common to the old subject ID and the new subject ID
				existingStudyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + newSubjectID + "' and @visitID='" + visitID + "']")
				studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + oldSubjectID + "' and @visitID='" + visitID + "']")
				for curStudyNode in studyNodes:
					studyID = curStudyNode.get("ID")
					existingStudyNode = None
					for curExistingStudyNode in existingStudyNodes:
						if studyID == curExistingStudyNode.get("ID"):
							existingStudyNode = curExistingStudyNode
					#for curExistingStudyNode in existingStudyNodes:
					if existingStudyNode == None: # no existing studies, just move the study over, changing all the IDs in subordinate fields
						print "study node " + existingStudyNode.get("ID") + " not found"
						episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + oldSubjectID + "' and @visitID='" + visitID + " and @studyID='" + studyID + "']")
						acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + oldSubjectID + "' and @visitID='" + visitID + " and @studyID='" + studyID + "']")
						dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + oldSubjectID + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "']")
						
						subjectURI = os.path.join(curStudyNode.get("projectID"), newSubjectID.zfill(DataTransferConstants.padZerosLength))
						visitURI = os.path.join(subjectURI, visitID)
						
						curStudyNode.set("subjectID", newSubjectID)
						curStudyNode.set("subjectURI", subjectURI)
						curStudyNode.set("visitURI", visitURI)

						studyURI = os.path.join(visitURI, studyID)
						for curNodeToChange in episodeNodes:
							curNodeToChange.set("subjectID", newSubjectID)
							curNodeToChange.set("subjectURI", subjectURI)
							curNodeToChange.set("visitURI", visitURI)
							curNodeToChange.set("studyURI", studyURI)
						
						for curNodeToChange in acquisitionNodes:
							episodeURI = os.path.join(studyURI, curNodeToChange.get("episodeID"))
							curNodeToChange.set("subjectID", newSubjectID)
							curNodeToChange.set("subjectURI", subjectURI)
							curNodeToChange.set("visitURI", visitURI)
							curNodeToChange.set("studyURI", studyURI)
							curNodeToChange.set("episodeURI", episodeURI)
						
						for curNodeToChange in dataResourceNodes:
							episodeURI = os.path.join(studyURI, curNodeToChange.get("episodeID"))
							acquisitionURI = os.path.join(episodeURI, curNodeToChange.get("acquisitionID"))
							curNodeToChange.set("subjectID", newSubjectID)
							curNodeToChange.set("subjectURI", subjectURI)
							curNodeToChange.set("visitURI", visitURI)
							curNodeToChange.set("studyURI", studyURI)
							curNodeToChange.set("episodeURI", episodeURI)
							curNodeToChange.set("acquisitionURI", acquisitionURI)
						# move the study from the old visit to the new visit
						sourceDir = os.path.join(databaseDirectory, projectName, 'data', str(oldSubjectID).zfill(DataTransferConstants.padZerosLength), visitID, studyID)
						destDir = os.path.join(databaseDirectory, projectName, 'data', str(newSubjectID).zfill(DataTransferConstants.padZerosLength), visitID, studyID)
						if verbose == True:
							print "mv " + sourceDir + " -> " + destDir
						shutil.move(sourceDir, destDir)
					else: # we have found this study in the new subject, so now we have to look for duplicate episodes
						episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + str(oldSubjectID) + "' and @visitID='" + visitID + "']")
						existingEpisodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + newSubjectID + "' and @visitID='" + visitID + "']")
						for curEpisodeNode in episodeNodes:
							episodeID = curEpisodeNode.get("ID")
							existingEpisodeNode = None
							for curExistingEpisodeNode in existingEpisodeNodes:
								if episodeID == curExistingEpisodeNode.get("ID"):
									existingEpisodeNode = curExistingEpisodeNode
							if existingEpisodeNode == None:
								# the study exists but the episode doesnt, move the episode directory in and change all the IDs
								acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + str(oldSubjectID) + "' and @visitID='" + visitID + "' and @episodeID='" + episodeID + "']")
								dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + str(oldSubjectID) + "' and @visitID='" + visitID + "' and @episodeID='" + episodeID + "']")
								
								subjectURI = os.path.join(curStudyNode.get("projectID"), newSubjectID.zfill(DataTransferConstants.padZerosLength))
								visitURI = os.path.join(subjectURI, visitID)
								studyURI = os.path.join(visitURI, studyID)
								episodeURI = os.path.join(studyURI, episodeID)
								
								curEpisodeNode.set("subjectID", newSubjectID)
								curEpisodeNode.set("subjectURI", subjectURI)
								curEpisodeNode.set("visitURI", visitURI)
								curEpisodeNode.set("studyURI", studyURI)

								for curNodeToChange in acquisitionNodes:
									curNodeToChange.set("subjectID", newSubjectID)
									curNodeToChange.set("subjectURI", subjectURI)
									curNodeToChange.set("visitURI", visitURI)
									curNodeToChange.set("studyURI", studyURI)
									curNodeToChange.set("episodeURI", episodeURI)
								
								for curNodeToChange in dataResourceNodes:
									acquisitionURI = os.path.join(episodeURI, curNodeToChange.get('acquisitionID'))
									curNodeToChange.set("subjectID", newSubjectID)
									curNodeToChange.set("subjectURI", subjectURI)
									curNodeToChange.set("visitURI", visitURI)
									curNodeToChange.set("studyURI", studyURI)
									curNodeToChange.set("episodeURI", episodeURI)
									curNodeToChange.set("acquisitionURI", acquisitionURI)
								# move the study from the old visit to the new visit
								sourceDir = os.path.join(databaseDirectory, projectName, 'data', str(oldSubjectID).zfill(DataTransferConstants.padZerosLength), visitID, studyID, episodeID)
								destDir = os.path.join(databaseDirectory, projectName, 'data', newSubjectID.zfill(DataTransferConstants.padZerosLength), visitID, studyID, episodeID)
								if verbose == True:
									print "mv " + sourceDir + " > " + destDir
								shutil.move(sourceDir, destDir)
							else: # the episode exists, so merge in any acquisitions underneath the episode
								acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + newSubjectID + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "' and @episodeID='" + episodeID + "']")
								existingAcquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + str(oldSubjectID) + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "' and @episodeID='" + episodeID + "']")
								# need to generate a new acquisition ID for the volume that is coming across
								nextAcquisitionID = -1
								for curExistingAcquisitionNode in existingAcquisitionNodes:
									if int(curExistingAcquisitionNode.get("ID")) >= nextAcquisitionID:
										nextAcquisitionID = int(curExistingAcquisitionNode.get("ID")) + 1
								for curAcquisitionNode in acquisitionNodes:
									dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + str(oldSubjectID) + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "' and @episodeID='" + episodeID + "' and @acquisitionID='" + curAcquisitionNode.get("ID") + "']")
									for curDataResourceNode in dataResourceNodes:
										curDataResourceNode.set("acquisitionID", str(nextAcquisitionID))
										curDataResourceNode.set("subjectID", newSubjectID)
									# move the episodes into the new episode now
									sourceDir = os.path.join(databaseDirectory, projectName, 'data', str(oldSubjectID).zfill(DataTransferConstants.padZerosLength), visitID, studyID, episodeID, curAcquisitionNode.get("ID").zfill(DataTransferConstants.padZerosLength))
									destDir = os.path.join(databaseDirectory, projectName, 'data', newSubjectID.zfill(DataTransferConstants.padZerosLength), visitID, studyID, episodeID, str(nextAcquisitionID).zfill(DataTransferConstants.padZerosLength))
									if verbose == True:
										print "mv " + sourceDir + " > " + destDir
									shutil.move(sourceDir, destDir)
									nextAcquisitionID = nextAcquisitionID + 1
									curAcquisitionNode.set("ID", str(nextAcquisitionID))
									# because the episode existed in the new subject ID, we need to remove the episode node from the XML
									projectXMLRoot.remove(curEpisodeNode)
							#if existingEpisodeNode == None:
						# this is where we have found the study in the new subject ID, we need to remove the old study node from the XML
						projectXMLRoot.remove(curStudyNode)
					#if existingStudyNode == None:
				#for curStudyNode in studyNodes:
				# the visit node has been found in the new subject ID, remove the visit node from the XML
				projectXMLRoot.remove(visitNode)
			#if curExistingVisitNode == None:
		#for curVisitNode in visitNodes:
		# we have finished processing the subject, remove the old subject node from the XML, remove the directory tree, save the XML file
		projectXMLRoot.remove(subjectNode)
		
		for curSubjectNode in projectControlsListRoot.iterchildren():
			if curSubjectNode.text == oldSubjectID:
				projectControlsListRoot.remove(curSubjectNode)
		for curSubjectNode in projectPatientsListRoot.iterchildren():
			if curSubjectNode.text == oldSubjectID:
				projectPatientsListRoot.remove(curSubjectNode)

		shutil.rmtree(os.path.join(databaseDirectory, projectName, 'data', str(oldSubjectID).zfill(DataTransferConstants.padZerosLength)))
		if verbose == True:
			print "rm " + os.path.join(databaseDirectory, projectName, 'data', str(oldSubjectID).zfill(DataTransferConstants.padZerosLength))
		projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
		
		nameHashMapping.deleteSubjectByID(oldSubjectID)
		
		#del nameHashMapping[int(oldSubjectID)]
		#nameHashMappingFile = file(os.path.join(databaseDirectory, projectName, "namehashmapping.txt"), 'w')
		#for curSubjectID in sorted(nameHashMapping.keys()):
		#	nameHashMappingFile.write(str(curSubjectID) + "," + nameHashMapping[curSubjectID]['hash'] + "," + nameHashMapping[curSubjectID]['name'] + "," + nameHashMapping[curSubjectID]['sex'] + "," + nameHashMapping[curSubjectID]['dob'] + "," + nameHashMapping[curSubjectID]['address'] + "\n")
		#nameHashMappingFile.close()
	#if len(subjectNode) == 0:
#def main():


#			else:
#				# simply rename all the visit ids
#				visitNode.set("ID", visitID)
#				studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + oldSubjectID + "' and @visitID='" + visitID + "']")
#				episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + oldSubjectID + "' and @visitID='" + visitID + "']")
#				acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + oldSubjectID + "' and @visitID='" + visitID + "']")
#				dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + oldSubjectID + "' and @visitID='" + visitID + "']")
#				
#				for curNodeToChange in studyNodes:
#					curNodeToChange.set("subjectID", newSubjectID)
#					curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), newSubjectID))
#				
#				for curNodeToChange in episodeNodes:
#					curNodeToChange.set("subjectID", newSubjectID)
#					curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), newSubjectID))
#					curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), visitID, curNodeToChange.get("studyID")))
#				
#				for curNodeToChange in acquisitionNodes:
#					curNodeToChange.set("subjectID", newSubjectID)
#					curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), newSubjectID))
#					curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), visitID, curNodeToChange.get("studyID")))
#					curNodeToChange.set("episodeURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), visitID, curNodeToChange.get("studyID"), curNodeToChange.get("episodeID")))
#				
#				for curNodeToChange in dataResourceNodes:
#					curNodeToChange.set("subjectID", newSubjectID)
#					curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), newSubjectID))
#					curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), visitID, curNodeToChange.get("studyID")))
#					curNodeToChange.set("episodeURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), visitID, curNodeToChange.get("studyID"), curNodeToChange.get("episodeID")))
#					curNodeToChange.set("acquisitionURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), visitID, curNodeToChange.get("studyID"), curNodeToChange.get("episodeID"), curNodeToChange.get("acquisitionID")))
#
#				# move the visit directory
#				print "mv " + os.path.join(databaseDirectory, projectName, 'data', oldSubjectID.zfill(DataTransferConstants.padZerosLength), visitID) + " > " + os.path.join(databaseDirectory, projectName, 'data', oldSubjectID.zfill(DataTransferConstants.padZerosLength), visitID)
#				shutil.move(os.path.join(databaseDirectory, projectName, 'data', oldSubjectID.zfill(DataTransferConstants.padZerosLength), visitID), \
#				os.path.join(databaseDirectory, projectName, 'data', newSubjectID.zfill(DataTransferConstants.padZerosLength), visitID))
#				projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
			#if len(existingVisitNode) > 0:
	#if len(subjectNode) == 0:
#def main():

#
#
#			# find the existing
#			newSubjectNodes = []
#			if oldSubjectID != None:
#				for curSubjectNode in subjectNodes:
#					if DataTransferPatterns.matchNumericPattern(curSubjectNode.get("ID"), oldSubjectID) == True:
#						newSubjectNodes.append(curSubjectNode)
#				subjectNodes = newSubjectNodes
#				del newSubjectNodes
#			newVisitNodes = []
#			for curSubjectNode in subjectNodes:
#				visitNodes = projectXMLRoot.xpath("/XCEDE/visit[@subjectID='" + curSubjectNode.get("ID") + "']")\
#				if visitID != None:
#					for curVisitNode in visitNodes:
#						if DataTransferPatterns.matchNumericPattern(curVisitNode.get("ID"), visitID) == True:
#							newVisitNodes.append(curVisitNode)
#				else:
#					for curVisitNode in visitNodes:
#						newVisitNodes.append(curVisitNode)
#			visitNodes = newVisitNodes
#			del newVisitNodes
#			for curVisitNode in visitNodes:
#				studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + curVisitNode.get("subjectID") + "' and @visitID='" + curVisitNode.get("ID") + "']")
#				if studyID != None:
#					for curStudyNode in StudyNodes:
#						if DataTransferPatterns.matchNumericPattern(curStudyNode.get("ID"), studyID) == True:
#							newStudyNodes.append(curStudyNode)
#				else:
#					for curStudyNode in StudyNodes:
#						newStudyNodes.append(curStudyNode)
#			StudyNodes = newStudyNodes
#			del newStudyNodes
#
#		projectControlsListRoot = projectXMLRoot.xpath("/XCEDE/dataResource[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='controls']")
#	

if __name__ == "__main__":
	main()
